<!-- The command-line parameters are:
	phase           NMTOKEN | "#ALL" (default) Select the phase for validation
	allow-foreign   "true" | "false" (default)   Pass non-Schematron elements to the generated stylesheet
	diagnose= true | false|yes|no    Add the diagnostics to the assertion test in reports (yes|no are obsolete)
	generate-paths=true|false|yes|no   generate the @location attribute with XPaths (yes|no are obsolete)
	sch.exslt.imports semi-colon delimited string of filenames for some EXSLT implementations          
	optimize        "visit-no-attributes"     Use only when the schema has no attributes as the context nodes
	generate-fired-rule "true"(default) | "false"  Generate fired-rule elements
-->

<xsl:stylesheet
	version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
	xmlns:isns="http://www.ato.gov.au/IS/2009/02"
	xmlns:i="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:tns="http://sbr.gov.au/comn/event.02.data"
    xmlns:sbdm="http://sbr.gov.au/comn/sbdm.02.data">


    <xsl:template match="/SWS_ProcessMessages">
        <ProcessMessages xmlns="http://www.ato.gov.au/IS/2009/02">
                <xsl:apply-templates select ="SWS_ProcessMessage"/>
        </ProcessMessages>
    </xsl:template>

    <xsl:template match="/EBMS_ProcessMessages">
        <tns:Event xmlns="http://www.ato.gov.au/IS/2009/02">
            <!--<xsl:apply-templates select ="EBMS_MaximumSeverity"/>-->
            <xsl:apply-templates select ="EBMS_ProcessMessage"/>
        </tns:Event>
    </xsl:template>
    <xsl:template match="/SBR_ProcessMessages">
        <sbdm:MessageEventItems xmlns="http://www.ato.gov.au/IS/2009/02">
            <xsl:apply-templates select ="SBR_ProcessMessage"/>
        </sbdm:MessageEventItems>
    </xsl:template>
    

    <xsl:template match ="SWS_ProcessMessage">
            <ProcessMessage xmlns="http://www.ato.gov.au/IS/2009/02">
                <CategoryCode>Syntax</CategoryCode>
                <isns:ErrorCode>
                    <xsl:value-of select="ErrorCode/text()" />
                </isns:ErrorCode>
                <Location>
                    <xsl:value-of select="Location/text()" />
                </Location>
                <isns:Description>
                    <xsl:value-of select="Description/text()" />
                </isns:Description>
                <xsl:if test="count(Parameters) = 1">
                    <isns:Parameters>
                        <xsl:for-each select="Parameters/Parameter">
                            <isns:Parameter>
                                <isns:Name>
                                    <xsl:value-of select="Name/text()" />
                                </isns:Name>
                                <isns:Value>
                                    <xsl:copy-of select ="Value/node()"/>
                                </isns:Value>
                            </isns:Parameter>
                        </xsl:for-each>
                    </isns:Parameters>
                </xsl:if>
                <PropertyName i:nil="true" />
                <xsl:choose>
                    <xsl:when test="string-length( Severity/text() ) &gt; 0">
                        <isns:Severity>
                            <xsl:value-of select="Severity/text()" />
                        </isns:Severity>
                    </xsl:when>
                    <xsl:otherwise>
                        <Severity>Error</Severity>
                    </xsl:otherwise>
                </xsl:choose>
            </ProcessMessage>
        </xsl:template>

    <!--<xsl:template   match ="ProcessMessage" xmlns="http://www.ato.gov.au/IS/2009/02">
        <tns:MaximumSeverity.Code/>
    </xsl:template>-->

    <xsl:template  match ="EBMS_ProcessMessage">
        <tns:MaximumSeverity.Code/>
        <tns:EventItems xmlns="http://www.ato.gov.au/IS/2009/02">
                <tns:EventItem>
                    <tns:Error.Code>
                        <xsl:value-of select="ErrorCode/text()" />
                    </tns:Error.Code>
                    <xsl:choose>
                        <xsl:when test="string-length( Severity/text() ) &gt; 0">
                            <tns:Severity.Code>
                                <xsl:value-of select="Severity/text()" />
                            </tns:Severity.Code>
                        </xsl:when>
                        <xsl:otherwise>
                            <tns:Severity.Code>Error</tns:Severity.Code>
                        </xsl:otherwise>
                    </xsl:choose>

                    <xsl:if test="string-length(Description/text()) &gt; 0">
                        <tns:Short.Description>
                            <xsl:value-of select="Description/text()" />
                        </tns:Short.Description>
                    </xsl:if>

                    <xsl:if test="string-length(LongDescription/text()) &gt; 0">
                        <tns:Detailed.Description>
                            <xsl:value-of select="LongDescription/text()" />
                        </tns:Detailed.Description>
                    </xsl:if>

                    <xsl:if test="count(Parameters) = 1">
                        <tns:Parameters>
                            <xsl:for-each select="Parameters/Parameter">
                                <tns:Parameter>
                                    <tns:Parameter.Identifier>
                                        <xsl:value-of select="Name/text()" />
                                    </tns:Parameter.Identifier>
                                    <tns:Parameter.Text>
                                        <xsl:copy-of select ="Value/node()"/>
                                    </tns:Parameter.Text>
                                </tns:Parameter>
                            </xsl:for-each>
                            <tns:Parameter>
                                <tns:Parameter.Identifier>RuleIdentifier</tns:Parameter.Identifier>
                                <tns:Parameter.Text>
                                    <!-- The $id is the DVR id in the diagnostics sch file. Strip out the D to leave the VR. -->
                                    <!-- Eg: DVR.SS.RIR.000001 -> VR.SS.RIR.000001 -->
                                    <!--<xsl:value-of select="substring($id, 2, string-length($id))" />-->
                                </tns:Parameter.Text>
                            </tns:Parameter>
                        </tns:Parameters>
                    </xsl:if>

                    <tns:Locations>
                        <tns:Location>
                            <tns:Location.Instance.Identifier/>
                            <tns:Location.Path.Text>
                                <xsl:value-of select="Location/text()" />
                            </tns:Location.Path.Text>
                        </tns:Location>
                    </tns:Locations>
                </tns:EventItem>
        </tns:EventItems>
    </xsl:template>


    <xsl:template match ="SBR_ProcessMessage">
        
        <sbdm:MessageEventItem>
            <sbdm:Message.Event.Item.Error.Code>
                <xsl:value-of select="Error.Code/text()" />
            </sbdm:Message.Event.Item.Error.Code>

            <xsl:choose>
                <xsl:when test="string-length( Severity.Code/text() ) &gt; 0">
                    <sbdm:Message.Event.Item.Severity.Code>
                        <xsl:value-of select="Severity.Code/text()" />
                    </sbdm:Message.Event.Item.Severity.Code>
                </xsl:when>
                <xsl:otherwise>
                    <sbdm:Message.Event.Item.Severity.Code>Error</sbdm:Message.Event.Item.Severity.Code>
                </xsl:otherwise>
            </xsl:choose>

            <xsl:if test="string-length( Description/text() ) &gt; 0">
                <sbdm:Message.Event.Item.Short.Description>
                    <xsl:value-of select="Description/text()" />
                </sbdm:Message.Event.Item.Short.Description>
            </xsl:if>

            <xsl:if test="string-length( LongDescription/text() ) &gt; 0">
                <sbdm:Message.Event.Item.Detailed.Description>
                    <xsl:value-of select="LongDescription/text()" />
                </sbdm:Message.Event.Item.Detailed.Description>
            </xsl:if>

            <xsl:if test="count(Parameters) = 1">
                <sbdm:Parameters>
                    <xsl:for-each select="Parameters/Parameter">
                        <sbdm:Parameter>
                            <sbdm:Message.Event.Item.Parameter.Identifier>
                                <xsl:value-of select="Name/text()" />
                            </sbdm:Message.Event.Item.Parameter.Identifier>
                            <sbdm:Message.Event.Item.Parameter.Text>
                                <xsl:copy-of select ="Value/node()"/>
                            </sbdm:Message.Event.Item.Parameter.Text>
                        </sbdm:Parameter>
                    </xsl:for-each>
                    <sbdm:Parameter>
                        <sbdm:Message.Event.Item.Parameter.Identifier>RuleIdentifier</sbdm:Message.Event.Item.Parameter.Identifier>
                        <sbdm:Message.Event.Item.Parameter.Text>
                            <!--<xsl:value-of select="substring($id, 2, string-length($id))" />-->
                            <!--<xsl:value-of select="$id" />-->
                        </sbdm:Message.Event.Item.Parameter.Text>
                    </sbdm:Parameter>
                  </sbdm:Parameters>
            </xsl:if>

            <sbdm:Locations>
                <sbdm:Location>
                    <sbdm:BusinessDocument.Sequence.Number/>
                    <sbdm:Message.Event.Item.Location.Path.Text>
                        <xsl:value-of select="Location/text()" />
                    </sbdm:Message.Event.Item.Location.Path.Text>
                </sbdm:Location>
            </sbdm:Locations>

        </sbdm:MessageEventItem>
    </xsl:template>
    


</xsl:stylesheet>