using System;
using System.Collections.Generic;
using Ato.EN.IntegrationServices.Document.DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationCTR {
    public partial class CTR2017Validator {
    
    #region Cross Form Rules - not supported yet

        #region VR.ATO.CTR.428027

        /*  VR.ATO.CTR.428027
                    Companies cannot claim CGT discount unless they are life insurance companies or friendly societies or the company type is corporate unit trusts or public trading trusts and cannot claim if the entity is head company of a consolidated group.

                Legacy Rule Format:
                    IF (([CTR49] <> SET ("63100","62400") AND 
                    [CTR52] <> SET ("054", "174")) OR ([CTR52] = SET ("054", "174") AND [CTR53] = "1")) AND [CGTS126] > 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS126] = CGTS:RP.Applied:gfagc.02.06:TaxConcession.CapitalGains.DiscountTotal.Amount
                    [CTR49] = CTR:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
                    [CTR52] = CTR:RP:pyde.02.26:OrganisationDetails.OrganisationType.Code
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code

            Technical Business Rule Format:
                    ((NotInSet(^CTR49, '"63100","62400"') AND NotInSet(^CTR52, '"054", "174"')) OR (InSet(^CTR52, '"054", "174"') AND ^CTR53 = '1')) AND ^CGTS126 > 0

        Data Elements:
            
                    RP:^CTR52 = tns:OrganisationDetails.OrganisationType.Code
            
                    RP:^CGTS126 = CGTS:RP.Applied:gfagc.02.06:TaxConcession.CapitalGains.DiscountTotal.Amount
            
                    RP:^CTR49 = tns:OrganisationDetails.OrganisationIndustry2006Extended.Code
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
                    */
        protected void VRATOCTR428027(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428027
            
            assertion = false; // = ((!(IsMatch(report.CTR49, @"^(63100|62400)$",RegexOptions.IgnoreCase)) && !(IsMatch(report.CTR52, @"^(054|174)$",RegexOptions.IgnoreCase)) || IsMatch(report.CTR52, @"^(054|174)$") && report.CTR53 == @"1") &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438052", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total CGT discount applied not applicable",
                    LongDescription = @"Companies cannot claim CGT discount unless they are life insurance companies or friendly societies or the company type is corporate unit trusts or public trading trusts and cannot claim if the entity is head company of a consolidated group",
                    //Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428027"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR49", Value = report.CTR49 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR52", Value = report.CTR52 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428027

        #region VR.ATO.CTR.428034

        /*  VR.ATO.CTR.428034
                    Losses schedule BP cannot be lodged with a consolidated Head company return.

                Legacy Rule Format:
                    IF ([CTR53] = "1") AND (COUNT(SCHEDULE = "LS") = 1)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code

            Technical Business Rule Format:
                    (^CTR53 = '1') AND (CountDocument('LS') = 1)

        Data Elements:
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
                    */
        protected void VRATOCTR428034(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428034
            
            assertion = false; // = (report.CTR53 == @"1" && @"LS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428034", Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule BP cannot be lodged with a consolidated Head company return",
                    LongDescription = @"A consolidated head company is not required to lodge a Losses schedule for unconsolidated entities",
                    //Location = "/xbrli:xbrl/tns:OrganisationDetails.TaxConsolidationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428034"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428034

        #region VR.ATO.CTR.428037

        /*  VR.ATO.CTR.428037
                    IEE specified income year to be completed when IEE schedule is attached

                Legacy Rule Format:
                    IF ([CTR55] = NULL) AND (COUNT(SCHEDULE = "IEE") > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR55] = CTR:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
                    (^CTR55 = NULL) AND (CountDocument('IEE') > 0)

        Data Elements:
            
                    RP:^CTR55 = tns:Elections.InterposedEntityElectionStatus.Year
                    */
        protected void VRATOCTR428037(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428037
            
            assertion = false; // = (report.CTR55 == null && @"IEE") > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428037", Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year to be completed when IEE schedule is attached",
                    LongDescription = @"An IEE schedule is attached: however the IEE specified income year has not been completed on the Company Income Tax Return",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428037"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR55", Value = report.CTR55.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428037

        #region VR.ATO.CTR.428039

        /*  VR.ATO.CTR.428039
                    Interposed Entity Election or Revocation schedule required where a revocation is being made

                Legacy Rule Format:
                    IF ([CTR56] = "R") AND (COUNT(SCHEDULE = "IEE") = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR56] = CTR:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
                    (^CTR56 = 'R') AND (CountDocument('IEE') = 0)

        Data Elements:
            
                    RP:^CTR56 = tns:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        protected void VRATOCTR428039(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428039
            
            assertion = false; // = (report.CTR56 == @"R" && @"IEE") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed Entity Election or Revocation schedule required where a revocation is being made",
                    LongDescription = @"The Interposed Entity Election code indicates a revocation is being made, however no schedule is attached",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428039"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR56", Value = report.CTR56 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428039

        #region VR.ATO.CTR.428040

        /*  VR.ATO.CTR.428040
                    Gross payments are required on Payment Summary Schedule, where amount entered at Gross Payments where ABN not quote

                Legacy Rule Format:
                    IF (([CTR58] > 0)  AND (COUNT(SCHEDULE = "PSS") = 0)) OR ((SUM[PSS19]) <> [CTR58])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR58] = CTR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    ((^CTR58 > 0)  AND (CountDocument('PSS') = 0)) OR ((Sum(^PSS19)) <> ^CTR58)

        Data Elements:
            
                    RP:^CTR58 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOCTR428040(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428040
            
            assertion = false; // = (report.CTR58.GetValueOrDefault() > 0 && @"PSS") == 0 || ( == null ? 0 : .Sum(f => f.)) != report.CTR58.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428040", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payments are required on Payment Summary Schedule, where amount entered at Gross Payments where ABN not quoted",
                    LongDescription = @"An amount has been entered at gross payment where ABN is not quoted and there are no corresponding amounts on the Payment Summary Schedule",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428040"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR58", Value = report.CTR58.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428040

        #region VR.ATO.CTR.428096

        /*  VR.ATO.CTR.428096
                    Capital Gains Tax schedule required, where net capital gains exceeds $10,000

                Legacy Rule Format:
                    IF (([CTR91] > 10000) AND 
                    ([CTR53] <> "2") AND 
                    (COUNT(SCHEDULE = "CGTS") = 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code
                    [CTR91] = CTR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    ((^CTR91 > 10000) AND (^CTR53 <> '2') AND (CountDocument('CGTS') = 0))

        Data Elements:
            
                    RP:^CTR91 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
                    */
        protected void VRATOCTR428096(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428096
            
            assertion = false; // = (report.CTR91.GetValueOrDefault() > 10000 && report.CTR53 != @"2" && @"CGTS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428379", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule required, where net capital gains exceeds $10,000",
                    LongDescription = @"When Net capital gains is in excess of $10,000, a Capital Gains Tax schedule must be attached",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428096"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR91", Value = report.CTR91.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428096

        #region VR.ATO.CTR.428098

        /*  VR.ATO.CTR.428098
                    Net capital gain in return must equal net capital gain in CGT schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([CTR91] <> [CGTS131])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [CTR91] = CTR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^CTR91 <> ^CGTS131)

        Data Elements:
            
                    RP:^CTR91 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^CGTS131 = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOCTR428098(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428098
            
            assertion = false; // = (@"CGTS") == 1 && report.CTR91.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430019", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain in return must equal net capital gain in CGT schedule",
                    LongDescription = @"Net capital gain amount on the company return  must equal the net capital gain amount on the CGT schedule",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428098"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR91", Value = report.CTR91.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428098

        #region VR.ATO.CTR.428099

        /*  VR.ATO.CTR.428099
                    Losses schedule must be attached where total of tax losses and net capital losses exceeds $100000

                Legacy Rule Format:
                    IF ([CTR175] + [CTR176] > 100000) AND 
                    ([CTR53] = NULLORBLANK) AND (COUNT(SCHEDULE = "LS") = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code
                    [CTR175] = CTR:RP:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [CTR176] = CTR:RP(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (^CTR175 + ^CTR176 > 100000) AND (^CTR53 = BLANK) AND (CountDocument('LS') = 0)

        Data Elements:
            
                    RP(Instant):^CTR176 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
            
                    RP:^CTR175 = tns:Tax.Losses.CarriedForward.Total.Amount
                    */
        protected void VRATOCTR428099(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428099
            
            assertion = false; // = (report.CTR175.GetValueOrDefault() + report.CTR176.GetValueOrDefault() > 100000 && string.IsNullOrWhiteSpace(report.CTR53) == true && @"LS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428099", Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached where total of tax losses and net capital losses exceeds $100000",
                    LongDescription = @"Where the total of tax losses and net capital losses exceeds $100000 and the company is not part of a consolidated group, a Losses schedule must be attached",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428099"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR175", Value = report.CTR175.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR176", Value = report.CTR176.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428099

        #region VR.ATO.CTR.428100

        /*  VR.ATO.CTR.428100
                    Consolidated losses schedule must be attached where Tax Losses and Net Capital Losses > $100000

                Legacy Rule Format:
                    IF ([CTR175] + [CTR176] > 100000) AND ([CTR53] = "1") AND (COUNT(SCHEDULE = "CGLS") = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code
                    [CTR175] = CTR:RP:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [CTR176] = CTR:RP(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (^CTR175 + ^CTR176 > 100000) AND (^CTR53 = '1') AND (CountDocument('CGLS') = 0)

        Data Elements:
            
                    RP(Instant):^CTR176 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
            
                    RP:^CTR175 = tns:Tax.Losses.CarriedForward.Total.Amount
                    */
        protected void VRATOCTR428100(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428100
            
            assertion = false; // = (report.CTR175.GetValueOrDefault() + report.CTR176.GetValueOrDefault() > 100000 && report.CTR53 == @"1" && @"CGLS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428100", Severity = ProcessMessageSeverity.Error,
                    Description = @"Consolidated losses schedule must be attached where Tax Losses and Net Capital Losses greater than $100000",
                    LongDescription = @"Consolidated losses schedule must be attached where total of tax losses and net capital losses exceeds $100000 and the Head company indicator is complete at Consolidated group status field",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428100"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR175", Value = report.CTR175.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR176", Value = report.CTR176.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428100

        #region VR.ATO.CTR.428101

        /*  VR.ATO.CTR.428101
                    Amount withheld required on Payment Summary schedule

                Legacy Rule Format:
                    IF ([CTR208] > 0) AND (((COUNT(SCHEDULE = "PSS") = 0) OR (SUM([PSS23]) <> [CTR208])))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR208] = CTR:RP:rvctc2.02.12:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount WHERE (rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.PaymentType.Code = "DNOABN") IN TUPLE(ctr.0008.lodge.req.02.00:ITPAYGWithholdingDetails)
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount

            Technical Business Rule Format:
                    (^CTR208 > 0) AND (((CountDocument('PSS') = 0) OR (Sum(^PSS23) <> ^CTR208)))

        Data Elements:
            
                    RP:^CTR208 = tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount WHERE [IncomeTax.PayAsYouGoWithholding.PaymentType.Code='DNOABN']
            
                    RP:^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    */
        protected void VRATOCTR428101(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428101
            
            assertion = false; // = (report.CTR208.GetValueOrDefault() > 0 && (@"PSS") == 0 || ( == null ? 0 : .Sum(f => f.)) != report.CTR208.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428101", Severity = ProcessMessageSeverity.Error,
                    Description = @"Amount withheld required on Payment Summary schedule",
                    LongDescription = @"An amount has been entered at tax withheld from payments where ABN no quoted in the calculation statement and on the Payment Summary schedule there are no corresponding amounts and the payment type is ""N""",
                    //Location = "/xbrli:xbrl/tns:ITPAYGWithholdingDetails" + OccurrenceIndex(report.ItpaygWithholdingDetailsCollection_DNOABN_RPOccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428101"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR208", Value = report.CTR208.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428101

        #region VR.ATO.CTR.428102

        /*  VR.ATO.CTR.428102
                    Companies are required to indicate whether continuity of majority ownership test passes for each year of loss

                Legacy Rule Format:
                    IF ([CTR53] = NULLORBLANK) AND ([CTR117] > 100000) AND (COUNT(SCHEDULE = "LS") = 1) AND ([LS15] = NULL) AND ([LS16] = NULL) AND ([LS17] = NULL) AND ([LS18] = NULL) AND ([LS19] = NULL) AND ([LS53] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code
                    [CTR117] = CTR:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
                    [LS15] = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS16] = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS17] = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS18] = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS19] = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS53] = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator

            Technical Business Rule Format:
                    (^CTR53 = BLANK) AND (^CTR117 > 100000) AND (CountDocument('LS') = 1) AND (^LS15 = NULL) AND (^LS16 = NULL) AND (^LS17 = NULL) AND (^LS18 = NULL) AND (^LS19 = NULL) AND (^LS53 = NULL)

        Data Elements:
            
                    RP:^CTR117 = tns:IncomeTax.Deduction.TaxLossesDeducted.Amount
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
            
                    RP:^LS15 = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS16 = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS17 = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS18 = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS19 = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS53 = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    */
        protected void VRATOCTR428102(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428102
            
            assertion = false; // = (string.IsNullOrWhiteSpace(report.CTR53) == true && report.CTR117.GetValueOrDefault() > 100000 && @"LS") == 1 &&  == null &&  == null &&  == null &&  == null &&  == null &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428102", Severity = ProcessMessageSeverity.Error,
                    Description = @"Companies are required to indicate whether continuity of majority ownership test passes for each year of loss",
                    LongDescription = @"Companies not part of the consolidated group that have Tax losses deducted of greater than $100.000 (Item 7 Label R) must complete Part B1 on the Losses schedule. Companies are required to indicate whether continuity of majority ownership test passed for each year of loss",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428102"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR117", Value = report.CTR117.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428102

        #region VR.ATO.CTR.428109

        /*  VR.ATO.CTR.428109
                    Credit for tax withheld-foreign resident withholding (excluding capital gains) must not be less than the amount present in the schedule.

                Legacy Rule Format:
                    IF (([CTR57] > 0) AND (COUNT(SCHEDULE = "PSS") = 0)) OR ([CTR207] < SUM[PSS22])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR57] = CTR:RP:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [CTR207] = CTR:RP:rvctc2.02.12:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount WHERE (rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.PaymentType.Code = "DFRW") IN TUPLE(ctr.0008.lodge.req.02.00:ITPAYGWithholdingDetails)
                    [PSS22] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount

            Technical Business Rule Format:
                    ((^CTR57 > 0) AND (CountDocument('PSS') = 0)) OR (^CTR207 < Sum(^PSS22))

        Data Elements:
            
                    RP:^CTR207 = tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount WHERE [IncomeTax.PayAsYouGoWithholding.PaymentType.Code='DFRW']
            
                    RP:^CTR57 = tns:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP:^PSS22 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                    */
        protected void VRATOCTR428109(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428109
            
            assertion = false; // = (report.CTR57.GetValueOrDefault() > 0 && @"PSS") == 0 || report.CTR207.GetValueOrDefault() < ( == null ? 0 : .Sum(f => f.)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438074", Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld-foreign resident withholding (excluding capital gains) must not be less than the amount present in the schedule.",
                    LongDescription = @"There is an amount present at Gross payments subject to foreign resident withholding (excluding capital gains) and Credit for tax withheld - foreign resident withholding (excluding capital gains) (Calculation Statement) on the return form does not equal the sum of the amounts on the Payment Summary schedule.",
                    //Location = "/xbrli:xbrl/tns:ITPAYGWithholdingDetails" + OccurrenceIndex(report.ItpaygWithholdingDetailsCollection_DFRW_RPOccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428109"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR57", Value = report.CTR57.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR207", Value = report.CTR207.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428109

        #region VR.ATO.CTR.428135

        /*  VR.ATO.CTR.428135
                    Consolidated losses schedule may be required

                Legacy Rule Format:
                    IF (([CTR117] > 100000) AND ([CTR53] = "1") AND (COUNT(SCHEDULE = "CGLS") = 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR53] = CTR:RP:pyde.02.03:OrganisationDetails.TaxConsolidationStatus.Code
                    [CTR117] = CTR:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
                    ((^CTR117 > 100000) AND (^CTR53 = '1') AND (CountDocument('CGLS') = 0))

        Data Elements:
            
                    RP:^CTR117 = tns:IncomeTax.Deduction.TaxLossesDeducted.Amount
            
                    RP:^CTR53 = tns:OrganisationDetails.TaxConsolidationStatus.Code
                    */
        protected void VRATOCTR428135(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428135
            
            assertion = false; // = (report.CTR117.GetValueOrDefault() > 100000 && report.CTR53 == @"1" && @"CGLS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428135", Severity = ProcessMessageSeverity.Error,
                    Description = @"Consolidated losses schedule may be required",
                    LongDescription = @"If the amount of Tax losses deducted is greater than $100000 and the Consolidated group status indicator is equal to ""1"" for head company then a Consolidated losses schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428135"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR117", Value = report.CTR117.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR53", Value = report.CTR53 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428135

        #region VR.ATO.CTR.428187

        /*  VR.ATO.CTR.428187
                    Total net capital losses C/F must equal Label V Net Capital Losses C/F in the Company Income Tax Form

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "CGLS") = 1) AND ([CGLS34] > 0) AND ([CTR176] <> [CGLS34])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGLS34] = CGLS:RP.TOT(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CTR176] = CTR:RP(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('CGLS') = 1) AND (^CGLS34 > 0) AND (^CTR176 <> ^CGLS34)

        Data Elements:
            
                    RP(Instant):^CTR176 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP(Instant):^CGLS34 = CGLS:RP.TOT(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOCTR428187(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428187
            
            assertion = false; // = (@"CGLS") == 1 &&  > 0 && report.CTR176.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428187", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total net capital losses C/F must equal Label V Net Capital Losses C/F in the Company Income Tax Form",
                    LongDescription = @"The total amount of Net capital losses carried forward to later income year on the Losses schedule for a consolidated group (Part A 10 Label V) must equal the amount of Net capital losses carried forward to later income year (Item 13 Label V) on the Company tax return",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428187"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR176", Value = report.CTR176.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428187

        #region VR.ATO.CTR.428190

        /*  VR.ATO.CTR.428190
                    Total of tax losses C/F to later income year in schedule must equal tax losses C/F to late year in the return

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND 
                    ([CTR175] <> [LS50]) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR175] = CTR:RP:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [LS50] = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND (^CTR175 <> ^LS50)

        Data Elements:
            
                    RP:^CTR175 = tns:Tax.Losses.CarriedForward.Total.Amount
            
                    RP:^LS50 = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    */
        protected void VRATOCTR428190(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428190
            
            assertion = false; // = (@"LS") == 1 && report.CTR175.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428190", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total of tax losses C/F to later income year in schedule must equal tax losses C/F to late year in the return",
                    LongDescription = @"Total of the tax losses carried forward to later income years in the losses schedule (Part A Label U) for unconsolidated entities must equal tax losses carried forward to later income years (Item 13 Label U) on the company Income Tax Return form",
                    //Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428190"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR175", Value = report.CTR175.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428190

        #region VR.ATO.CTR.428191

        /*  VR.ATO.CTR.428191
                    Net capital losses carried forward incorrect

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND ([CTR176] <> [LS52]) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR176] = CTR:RP(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [LS52] = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND (^CTR176 <> ^LS52)

        Data Elements:
            
                    RP(Instant):^CTR176 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP(Instant):^LS52 = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOCTR428191(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428191
            
            assertion = false; // = (@"LS") == 1 && report.CTR176.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward incorrect",
                    LongDescription = @"Net capital losses carried forward to later income years - total in Losses schedule for unconsolidated entities is must equal to the Net capital losses carried forward to later income year, Losses information, (Item 13 Label V) on the main income tax return",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428191"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR176", Value = report.CTR176.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428191

        #region VR.ATO.CTR.428234

        /*  VR.ATO.CTR.428234
                    Main entity cannot interpose itself

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (COUNT(SCHEDULE = "IEE") > 0) AND (RP:entity.identifier.TFN = ANY OCCURRENCE OF ([IEE50]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.xx.xx:FamilyTrust)

            Technical Business Rule Format:
                    (^TFN <> BLANK) AND (CountDocument('IEE') > 0) AND (AnyOccurrence(^IEE50, ^IEE50 = ^TFN))

        Data Elements:
            
                    ^IEE50 = IEE:RP:pyid.66.66:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.66.66:FamilyTrust)
            
                    ^TFN = RP
                    */
        protected void VRATOCTR428234(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428234
            
            assertion = false; // = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && @"IEE") > 0 && ( == null ? false : .Any(IEE50Repeat =>  == report.RPIdentifierTFN)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428234", Severity = ProcessMessageSeverity.Error,
                    Description = @"Main entity cannot interpose itself",
                    LongDescription = @"An Entity cannot interpose itself, therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the entity making the election",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428234"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428234

        #region VR.ATO.CTR.428240

        /*  VR.ATO.CTR.428240
                    Revocation date of the Interposed Entity Election is invalid

                Legacy Rule Format:
                    IF (CTR:RP:pyin.xx.xx:Report.TargetFinancial.Year <> NULL) AND (COUNT(SCHEDULE = "IEE") > 0) AND (ANY OCCURRENCE OF ([IEE116] = "R") AND ([IEE82] <> CTR:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE82] = IEE:RP:rvctc3.xx.xx:Elections.Revocation.Year
                    [IEE116] = IEE:RP:rvctc3.xx.xx:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
                    (^CTR2 <> NULL) AND (CountDocument('IEE') > 0) AND ((AnyOccurrence(^IEE116, ^IEE116 = 'R')) AND (^IEE82 <> ^CTR2))

        Data Elements:
            
                    RP:^CTR2 = tns:Report.TargetFinancial.Year
            
                    RP:^IEE82 = IEE:RP:rvctc3.66.66:Elections.Revocation.Year
            
                    RP:^IEE116 = IEE:RP:rvctc3.66.66:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        protected void VRATOCTR428240(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428240
            
            assertion = false; // = (report.CTR2 != null && @"IEE") > 0 && (( == null ? false : .Any(IEE116Repeat =>  == @"R")) &&  != report.CTR2.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428240", Severity = ProcessMessageSeverity.Error,
                    Description = @"Revocation date of the Interposed Entity Election is invalid",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged",
                    //Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428240"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR2", Value = report.CTR2.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428240

        #region VR.ATO.CTR.428278

        /*  VR.ATO.CTR.428278
                    If there is an amount greater than zero at 'Gross payments subject to foreign resident withholding (excluding capital gains)' a Non-individual Payment Summary schedule must be present, and sum of the Gross payments subject to foreign resident withholding (excluding capital gains) on the schedule must equal the amount on the main form.

                Legacy Rule Format:
                    IF (([CTR57] > 0) AND (COUNT(SCHEDULE = "PSS") = 0)) OR ((SUM[PSS18]) <> [CTR57])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR57] = CTR:RP:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
                    ((^CTR57 > 0) AND (CountDocument('PSS') = 0)) OR ((Sum(^PSS18)) <> ^CTR57)

        Data Elements:
            
                    RP:^CTR57 = tns:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    */
        protected void VRATOCTR428278(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428278
            
            assertion = false; // = (report.CTR57.GetValueOrDefault() > 0 && @"PSS") == 0 || ( == null ? 0 : .Sum(f => f.)) != report.CTR57.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438075", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payments are required on Payment Summary Schedule, where amount entered at Gross payments subject to foreign resident withholding (excluding capital gains).",
                    LongDescription = @"An amount has been entered at gross payment where Gross payments are subject to foreign resident withholding (excluding capital gains) and there is no corresponding amounts on the Payment Summary Schedule.",
                    //Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428278"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR57", Value = report.CTR57.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428278

        #region VR.ATO.CTR.428352

        /*  VR.ATO.CTR.428352
                    If Offshore banking unit adjustment is present or International related party dealings/pricing question are present and equal to true or Thin capitalisation question is present and equal to true.  An International Dealings Schedule is not present.

                Legacy Rule Format:
                    IF ([CTR113] <> NULL OR [CTR194] = TRUE OR [CTR195] = TRUE OR [CTR196] = TRUE) 
                        AND
                        (COUNT(SCHEDULE = "IDS") = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR113] = CTR:RP:gfagc.02.00:TaxConcession.OffshoreBankingUnitAdjustment.Amount
                    [CTR194] = CTR:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    [CTR195] = CTR:RP:rvctc4.02.03:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
                    [CTR196] = CTR:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
                    (^CTR113 <> NULL OR ^CTR194 = TRUE OR ^CTR195 = TRUE OR ^CTR196 = TRUE)     AND    (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP:^CTR113 = tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount
            
                    RP:^CTR194 = tns:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            
                    RP:^CTR195 = tns:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
            
                    RP:^CTR196 = tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOCTR428352(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428352
            
            assertion = false; // = ((report.CTR113 != null || report.CTR194 == true || report.CTR195 == true || report.CTR196 == true) && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428352", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings schedule required",
                    LongDescription = @"There is an amount present at Offshore banking unit adjustment or there is a 'yes' response to any of the questions concerning whether the value of dealings with international related parties is greater than $2 million or whether the entity had either a direct or indirect interest in a foreign trust, foreign company, controlled foreign entity or transferor trust or there is a 'yes' response to the question ""Did the thin capitalisation provisions apply"" and an International Dealings schedule is not attached.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428352"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR113", Value = report.CTR113.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR194", Value = GetValueOrEmpty(report.CTR194) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR195", Value = GetValueOrEmpty(report.CTR195) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR196", Value = GetValueOrEmpty(report.CTR196) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428352

        #region VR.ATO.CTR.428353

        /*  VR.ATO.CTR.428353
                    If 'Interest expenses overseas', 'Royalty expenses overseas', 'Section 46fa deduction for flowon dividends' and 'Offshore banking unit adjustment' are not present and there is a "no" response to the questions concerning whether the value of dealings with international related parties is greater than $2 million and whether the entity had either a direct or indirect interest in a foreign trust, foreign company, controlled foreign entity or transferor trust and there is a "no" response to the question 'Did the thin capitalisation provisions affect you' and an International Dealings schedule is attached.

                Legacy Rule Format:
                    IF (([CTR81] = NULL OR [CTR81] = 0) AND ([CTR366] = NULL OR [CTR366] = 0) AND ([CTR98] =NULL OR [CTR98] = 0)) AND [CTR113] = NULL AND ([CTR194] = FALSE AND [CTR195] = FALSE AND [CTR196] = FALSE) 
                        AND (COUNT(SCHEDULE = "IDS") = 1)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR81] = CTR:RP.JFOREIGN:bafpr2.02.17:Expense.Operating.Interest.Amount
                    [CTR98] = CTR:RP:bafpr1.02.02:Expense.FlowOnDividendDeduction.Amount
                    [CTR113] = CTR:RP:gfagc.02.00:TaxConcession.OffshoreBankingUnitAdjustment.Amount
                    [CTR194] = CTR:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    [CTR195] = CTR:RP:rvctc4.02.03:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
                    [CTR196] = CTR:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    [CTR366] = CTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    ((^CTR81 = NULL OR ^CTR81 = 0) AND (^CTR366 = NULL OR ^CTR366 = 0) AND (^CTR98 =NULL OR ^CTR98 = 0)) AND ^CTR113 = NULL AND (^CTR194 = FALSE AND ^CTR195 = FALSE AND ^CTR196 = FALSE)     AND (CountDocument('IDS') = 1)

        Data Elements:
            
                    RP:^CTR113 = tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount
            
                    RP.JFOREIGN:^CTR81 = tns:Expense.Operating.Interest.Amount
            
                    RP:^CTR98 = tns:Expense.FlowOnDividendDeduction.Amount
            
                    RP:^CTR194 = tns:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            
                    RP:^CTR195 = tns:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
            
                    RP:^CTR196 = tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            
                    RP.JFOREIGN:^CTR366 = tns:Expense.Royalties.Amount
                    */
        protected void VRATOCTR428353(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428353
            
            assertion = false; // = ((report.CTR81 == null || report.CTR81 == 0) && (report.CTR366 == null || report.CTR366 == 0) && (report.CTR98 == null || report.CTR98 == 0) && report.CTR113 == null && (report.CTR194 == false && report.CTR195 == false && report.CTR196 == false) && @"IDS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430232", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings schedule not required",
                    LongDescription = @"If 'Interest expenses overseas', 'Royalty expenses overseas', 'Section 46fa deduction for flowon dividends' and 'Offshore banking unit adjustment' are not present and there is a ""no"" response to the questions concerning whether the value of dealings with international related parties is greater than $2 million and whether the entity had either a direct or indirect interest in a foreign trust, foreign company, controlled foreign entity or transferor trust and there is a ""no"" response to the question 'Did the thin capitalisation provisions affect you' and an International Dealings schedule is attached.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428353"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR81", Value = report.CTR81.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR366", Value = report.CTR366.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR98", Value = report.CTR98.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR113", Value = report.CTR113.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR194", Value = GetValueOrEmpty(report.CTR194) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR195", Value = GetValueOrEmpty(report.CTR195) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR196", Value = GetValueOrEmpty(report.CTR196) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428353

        #region VR.ATO.CTR.428354

        /*  VR.ATO.CTR.428354
                    Where R&D labels have been completed a Research and Development Tax Incentive schedule must be attached.

                Legacy Rule Format:
                    IF ([CTR97] <> NULL OR [CTR328] <> NULL OR [CTR210] <> NULL) AND (COUNT(SCHEDULE = "RDTIS") = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR97] = CTR:RP:gfagc.02.05:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount
                    [CTR210] = CTR:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
                    [CTR328] = CTR:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    (^CTR97 <> NULL OR ^CTR328 <> NULL OR ^CTR210 <> NULL) AND (CountDocument('RDTIS') = 0)

        Data Elements:
            
                    RP:^CTR97 = tns:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount
            
                    RP:^CTR210 = tns:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^CTR328 = tns:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
                    */
        protected void VRATOCTR428354(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428354
            
            assertion = false; // = ((report.CTR97 != null || report.CTR328 != null || report.CTR210 != null) && @"RDTIS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428354", Severity = ProcessMessageSeverity.Error,
                    Description = @"Research and Development Tax Incentive schedule must be attached",
                    LongDescription = @"Where R&D labels have been completed a Research and Development Tax Incentive schedule must be attached.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428354"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR97", Value = report.CTR97.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR328", Value = report.CTR328.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR210", Value = report.CTR210.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428354

        #region VR.ATO.CTR.428355

        /*  VR.ATO.CTR.428355
                    R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.

                Legacy Rule Format:
                    IF [CTR97] <> [RDTIS3] AND (COUNT(SCHEDULE = "RDTIS") = 1) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR97] = CTR:RP:gfagc.02.05:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount
                    [RDTIS3] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.ResearchAndDevelopmentAccountingExpenditureAddBack.Amount

            Technical Business Rule Format:
                    ^CTR97 <> ^RDTIS3 AND (CountDocument('RDTIS') = 1)

        Data Elements:
            
                    RP:^CTR97 = tns:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount
            
                    RP:^RDTIS3 = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.ResearchAndDevelopmentAccountingExpenditureAddBack.Amount
                    */
        protected void VRATOCTR428355(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428355
            
            assertion = false; // = (report.CTR97.GetValueOrDefault() !=  && @"RDTIS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428355", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D label mismatch",
                    LongDescription = @"R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.ResearchAndDevelopment.WriteBackOfAccountingExpenditure.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428355"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR97", Value = report.CTR97.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428355

        #region VR.ATO.CTR.428398

        /*  VR.ATO.CTR.428398
                    R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.

                Legacy Rule Format:
                    IF [CTR328] <> [RDTIS48] AND (COUNT(SCHEDULE = "RDTIS") = 1) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR328] = CTR:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
                    [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    ^CTR328 <> ^RDTIS48 AND (CountDocument('RDTIS') = 1)

        Data Elements:
            
                    RP:^CTR328 = tns:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            
                    RP:^RDTIS48 = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
                    */
        protected void VRATOCTR428398(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428398
            
            assertion = false; // = (report.CTR328.GetValueOrDefault() !=  && @"RDTIS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428355", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D label mismatch",
                    LongDescription = @"R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428398"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR328", Value = report.CTR328.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428398

        #region VR.ATO.CTR.428399

        /*  VR.ATO.CTR.428399
                    R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.

                Legacy Rule Format:
                    IF [CTR210] <> [RDTIS46] AND (COUNT(SCHEDULE = "RDTIS") = 1) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR210] = CTR:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
                    [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
                    ^CTR210 <> ^RDTIS46 AND (CountDocument('RDTIS') = 1)

        Data Elements:
            
                    RP:^CTR210 = tns:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^RDTIS46 = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
                    */
        protected void VRATOCTR428399(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428399
            
            assertion = false; // = (report.CTR210.GetValueOrDefault() !=  && @"RDTIS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428355", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D label mismatch",
                    LongDescription = @"R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428399"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR210", Value = report.CTR210.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428399

        #region VR.ATO.CTR.428400

        /*  VR.ATO.CTR.428400
                    R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.

                Legacy Rule Format:
                    IF [CTR321] <> [RDTIS31] AND (COUNT(SCHEDULE = "RDTIS") = 1) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR321] = CTR:RP:rvctc1.02.08:IncomeTax.ResearchAndDevelopmentRecoupment.Amount
                    [RDTIS31] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.GovernmentGrantsRecoupmentsClawback.Amount

            Technical Business Rule Format:
                    ^CTR321 <> ^RDTIS31 AND (CountDocument('RDTIS') = 1)

        Data Elements:
            
                    RP:^CTR321 = tns:IncomeTax.ResearchAndDevelopmentRecoupment.Amount
            
                    RP:^RDTIS31 = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.GovernmentGrantsRecoupmentsClawback.Amount
                    */
        protected void VRATOCTR428400(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428400
            
            assertion = false; // = (report.CTR321.GetValueOrDefault() !=  && @"RDTIS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428355", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D label mismatch",
                    LongDescription = @"R&D labels do not equal their matching labels in the Research and Development Tax Incentive schedule.",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.ResearchAndDevelopmentRecoupment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428400"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR321", Value = report.CTR321.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428400

        #region VR.ATO.CTR.428404

        /*  VR.ATO.CTR.428404
                    An amended R&D tax incentive schedule may only be attached to an amended main form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RDTIS") = 1) AND [RDTIS1] = TRUE AND [CTR298] <> TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR298] = CTR:RP:pyin.02.28:Report.Amendment.Indicator
                    [RDTIS1] = RDTIS:RP:pyin.02.07:Report.Amendment.Indicator

            Technical Business Rule Format:
                    (CountDocument('RDTIS') = 1) AND ^RDTIS1 = TRUE AND ^CTR298 <> TRUE

        Data Elements:
            
                    RP:^CTR298 = tns:Report.Amendment.Indicator
            
                    RP:^RDTIS1 = RDTIS:RP:pyin.02.07:Report.Amendment.Indicator
                    */
        protected void VRATOCTR428404(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428404
            
            assertion = false; // = (@"RDTIS") == 1 &&  == true && report.CTR298 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428404", Severity = ProcessMessageSeverity.Error,
                    Description = @"An amended R&D tax incentive schedule may only be attached to an amended main form",
                    //Location = "/xbrli:xbrl/tns:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428404"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR298", Value = GetValueOrEmpty(report.CTR298) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428404

        #region VR.ATO.CTR.428405

        /*  VR.ATO.CTR.428405
                    If there is a 'Yes' response to 'Did the thin capitalisation rules apply to you?' on the IDS then a 'Yes' response is required at 'Did the thin capitalisation provisions apply as outlined in the instructions?' on the company return form

                Legacy Rule Format:
                    IF ([CTR196] = FALSE) AND ([IDS220] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR196] = CTR:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    [IDS220] = IDS:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
                    (^CTR196 = FALSE) AND (^IDS220 = TRUE)

        Data Elements:
            
                    RP:^CTR196 = tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            
                    RP:^IDS220 = IDS:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOCTR428405(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428405
            
            assertion = false; // = (report.CTR196 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Check response to Thin Capitalisation question",
                    LongDescription = @"If there is a 'True' response to 'Did the thin capitalisation rules apply to you?' on the IDS then a 'True' response is required at 'Did the thin capitalisation provisions apply as outlined in the instructions?' on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428405"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR196", Value = GetValueOrEmpty(report.CTR196) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428405

        #region VR.ATO.CTR.428406

        /*  VR.ATO.CTR.428406
                    Where there is a Royalties deduction amount present on the IDS, there must also be an amount at Royalty expenses overseas, or Gross payments subject to foreign resident withholding (excluding capital gains) on the company return form.

                Legacy Rule Format:
                    IF ([CTR57] = NULL OR [CTR57] = 0) AND (([CTR366] = NULL OR [CTR366] = 0) AND ([IDS22] > 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR57] = CTR:RP:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [CTR366] = CTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount

            Technical Business Rule Format:
                    (^CTR57 = NULL OR ^CTR57 = 0) AND ((^CTR366 = NULL OR ^CTR366 = 0) AND (^IDS22 > 0))

        Data Elements:
            
                    RP:^CTR57 = tns:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP.JFOREIGN:^CTR366 = tns:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOCTR428406(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428406
            
            assertion = false; // = ((report.CTR57 == null || report.CTR57 == 0) && ((report.CTR366 == null || report.CTR366 == 0) &&  > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438069", Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty expenses overseas or Foreign resident withholding expenses (excluding capital gains) required.",
                    LongDescription = @"If a Royalties deduction amount has been completed on the IDS then Foreign resident withholding (excluding capital gains) expenses or Royalty expenses overseas must be greater than zero on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428406"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR57", Value = report.CTR57.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR366", Value = report.CTR366.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428406

        #region VR.ATO.CTR.428408

        /*  VR.ATO.CTR.428408
                    If there is an amount at question 6a, label C on the IDS it must be less than or equal to the amount at question 6 label U on the company return form

                Legacy Rule Format:
                    IF ([CTR366] >  0) AND ([CTR366] < [IDS22])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR366] = CTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount

            Technical Business Rule Format:
                    (^CTR366 >  0) AND (^CTR366 < ^IDS22)

        Data Elements:
            
                    RP.JFOREIGN:^CTR366 = tns:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOCTR428408(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428408
            
            assertion = false; // = (report.CTR366.GetValueOrDefault() > 0 && report.CTR366.GetValueOrDefault() < );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalties deductions incorrect",
                    LongDescription = @"If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the Company return form",
                    //Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428408"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR366", Value = report.CTR366.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428408

        #region VR.ATO.CTR.428409

        /*  VR.ATO.CTR.428409
                    Where there is a Rent/leasing expenditure amount present on the IDS, there must also be an amount at either Lease expenses overseas or Rent expenses on the company return form

                Legacy Rule Format:
                    IF (([CTR78] = NULL OR [CTR78] = 0) AND ([CTR79] = NULL OR [CTR79] = 0)) AND ([IDS31] > 0) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR78] = CTR:RP.JFOREIGN:bafpr2.02.00:Expense.Operating.Lease.Amount
                    [CTR79] = CTR:RP:bafpr2.02.16:Expense.Operating.Rent.Amount
                    [IDS31] = IDS:RP.RentLease:bafpr1.02.16:Expense.Total.Amount

            Technical Business Rule Format:
                    ((^CTR78 = NULL OR ^CTR78 = 0) AND (^CTR79 = NULL OR ^CTR79 = 0)) AND (^IDS31 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^CTR78 = tns:Expense.Operating.Lease.Amount
            
                    RP:^CTR79 = tns:Expense.Operating.Rent.Amount
            
                    RP:^IDS31 = IDS:RP.RentLease:bafpr1.02.16:Expense.Total.Amount
                    */
        protected void VRATOCTR428409(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428409
            
            assertion = false; // = ((report.CTR78 == null || report.CTR78 == 0) && (report.CTR79 == null || report.CTR79 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Lease expenses overseas or Rent expenses are required",
                    LongDescription = @"Where there is a Rent/leasing expenditure amount present on the IDS, there must also be an amount at either Lease expenses overseas or Rent expenses on the company return form",
                    //Location = "/xbrli:xbrl/tns:Expense.Operating.Lease.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428409"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR78", Value = report.CTR78.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR79", Value = report.CTR79.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428409

        #region VR.ATO.CTR.428410

        /*  VR.ATO.CTR.428410
                    Where there is a Rent/leasing revenue amount present on the IDS, there must also be an amount at Gross rent and other leasing and hiring income amount on the company return form.

                Legacy Rule Format:
                    IF ([CTR65] = NULL OR [CTR65] = 0) AND ([IDS32] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR65] = CTR:RP:bafpr1.02.02:Income.RentLeasingHiringGross.Amount
                    [IDS32] = IDS:RP.RentLease:bafpr1.02.26:Revenue.Total.Amount

            Technical Business Rule Format:
                    (^CTR65 = NULL OR ^CTR65 = 0) AND (^IDS32 > 0)

        Data Elements:
            
                    RP:^CTR65 = tns:Income.RentLeasingHiringGross.Amount
            
                    RP:^IDS32 = IDS:RP.RentLease:bafpr1.02.26:Revenue.Total.Amount
                    */
        protected void VRATOCTR428410(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428410
            
            assertion = false; // = ((report.CTR65 == null || report.CTR65 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and other leasing and hiring income required",
                    LongDescription = @"If Rent/leasing revenue has been completed on the IDS then Gross rent and other leasing and hiring income must be greater than zero on the company return form",
                    //Location = "/xbrli:xbrl/tns:Income.RentLeasingHiringGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428410"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR65", Value = report.CTR65.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428410

        #region VR.ATO.CTR.428412

        /*  VR.ATO.CTR.428412
                    Where there is an interest revenue amount present on the IDS, there must also be an amount at Gross Interest or Gross payments subject to foreign resident withholding (excluding capital gains) on the company return form.

                Legacy Rule Format:
                    IF (([CTR57] = NULL OR [CTR57] = 0) AND ([CTR64] = NULL OR [CTR64] = 0)) AND ([IDS103] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR57] = CTR:RP:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [CTR64] = CTR:RP:bafpr2.02.02:Income.Interest.Gross.Amount
                    [IDS103] = IDS:RP.OtherInterest:bafpr1.02.26:Revenue.Total.Amount

            Technical Business Rule Format:
                    ((^CTR57 = NULL OR ^CTR57 = 0) AND (^CTR64 = NULL OR ^CTR64 = 0)) AND (^IDS103 > 0)

        Data Elements:
            
                    RP:^CTR57 = tns:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP:^CTR64 = tns:Income.Interest.Gross.Amount
            
                    RP:^IDS103 = IDS:RP.OtherInterest:bafpr1.02.26:Revenue.Total.Amount
                    */
        protected void VRATOCTR428412(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428412
            
            assertion = false; // = ((report.CTR57 == null || report.CTR57 == 0) && (report.CTR64 == null || report.CTR64 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438070", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross interest or Gross payments subject to foreign resident withholding (excluding capital gains) required.",
                    LongDescription = @"If Interest revenue has been completed on the IDS then Gross interest or Gross payments subject to foreign resident withholding (excluding capital gains) must be greater than zero on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428412"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR57", Value = report.CTR57.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR64", Value = report.CTR64.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428412

        #region VR.ATO.CTR.428413

        /*  VR.ATO.CTR.428413
                    The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or greater than the sum of amounts at Listed country and Unlisted country and equal to or less than the sum of amounts at Listed country, Unlisted country and Transferor trust on the company return form.

                Legacy Rule Format:
                    IF ([CTR156] > 0 OR [CTR158] > 0 OR [CTR159] > 0) AND ((([IDS191] + [IDS192] + [IDS193]) < ([CTR156] + [CTR158])) OR (([IDS191] + [IDS192] + [IDS193]) > ([CTR156] + [CTR158] + [CTR159])))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR156] = CTR:RP.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [CTR158] = CTR:RP.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [CTR159] = CTR:RP:rvctc4.02.00:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount
                    [IDS191] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
                    [IDS192] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
                    [IDS193] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount

            Technical Business Rule Format:
                    (^CTR156 > 0 OR ^CTR158 > 0 OR ^CTR159 > 0) AND (((^IDS191 + ^IDS192 + ^IDS193) < (^CTR156 + ^CTR158)) OR ((^IDS191 + ^IDS192 + ^IDS193) > (^CTR156 + ^CTR158 + ^CTR159)))

        Data Elements:
            
                    RP:^CTR159 = tns:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount
            
                    RP.CFCLC:^CTR156 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.CFCUC:^CTR158 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.CFCUC:^IDS191 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
            
                    RP.CFCUC:^IDS192 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
            
                    RP.CFCUC:^IDS193 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
                    */
        protected void VRATOCTR428413(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428413
            
            assertion = false; // = ((report.CTR156.GetValueOrDefault() > 0 || report.CTR158.GetValueOrDefault() > 0 || report.CTR159.GetValueOrDefault() > 0) && ( +  +  < report.CTR156.GetValueOrDefault() + report.CTR158.GetValueOrDefault() ||  +  +  > report.CTR156.GetValueOrDefault() + report.CTR158.GetValueOrDefault() + report.CTR159.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430222", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the International Dealing Schedule should be equal to or greater than the sum of amounts at Listed country, Unlisted country and equal to or less than the sum of amounts at Listed country,Unlisted country and Transferor trust on the company return form.",
                    //Location = "/xbrli:xbrl/tns:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428413"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR156", Value = report.CTR156.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR158", Value = report.CTR158.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR159", Value = report.CTR159.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428413

        #region VR.ATO.CTR.428414

        /*  VR.ATO.CTR.428414
                    CFCs attributable income - Listed countries amount on the International Dealings Schedule (IDS) must be equal to or less than Attributed foreign income - Listed country on the parent Company Tax Return (CTR)

                Legacy Rule Format:
                    IF ([CTR156] > 0) AND ([IDS188] > [CTR156])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR156] = CTR:RP.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [IDS188] = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount

            Technical Business Rule Format:
                    (^CTR156 > 0) AND (^IDS188 > ^CTR156)

        Data Elements:
            
                    RP.CFCLC:^CTR156 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.CFCLC:^IDS188 = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    */
        protected void VRATOCTR428414(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428414
            
            assertion = false; // = (report.CTR156.GetValueOrDefault() > 0 &&  > report.CTR156.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income amount is incorrect",
                    LongDescription = @"CFCs attributable income - Listed countries amount on the International Dealings Schedule (IDS) must be equal to or less than Attributed foreign income - Listed country on the parent Company Tax Return (CTR)",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428414"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR156", Value = report.CTR156.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428414

        #region VR.ATO.CTR.428415

        /*  VR.ATO.CTR.428415
                    Amounts are showing at attributable income of controlled foreign companies and controlled foreign trusts question 22 on IDS but there is no 'yes' response to the question concerning whether you have an overseas branch or a direct or indirect interest in a foreign trust, foreign company, controlled foreign entity or transferor trust on the company return form (overseas interests label Z).

                Legacy Rule Format:
                    IF [CTR195] = FALSE AND ([IDS191] > 0 OR [IDS192] > 0 OR [IDS193] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR195] = CTR:RP:rvctc4.02.03:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
                    [IDS191] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
                    [IDS192] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
                    [IDS193] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount

            Technical Business Rule Format:
                    ^CTR195 = FALSE AND (^IDS191 > 0 OR ^IDS192 > 0 OR ^IDS193 > 0)

        Data Elements:
            
                    RP:^CTR195 = tns:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator
            
                    RP:^IDS191 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
            
                    RP:^IDS192 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
            
                    RP:^IDS193 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
                    */
        protected void VRATOCTR428415(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428415
            
            assertion = false; // = (report.CTR195 == false && ( > 0 ||  > 0 ||  > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428415", Severity = ProcessMessageSeverity.Error,
                    Description = @"Response required at overseas interests question",
                    LongDescription = @"If assessable income of controlled foreign companies and controlled foreign trusts under sections 456, 457 or 459A has been completed on the IDS then the response to the Overseas interests question on the company return form must be 'True'.",
                    //Location = "/xbrli:xbrl/tns:InternationalDealings.DirectOrIndirectOverseasInterest.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428415"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR195", Value = GetValueOrEmpty(report.CTR195) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428415

        #region VR.ATO.CTR.428416

        /*  VR.ATO.CTR.428416
                    There is an amount at Listed country or Unlisted country or Transferor trust on the company tax return form and the number of controlled foreign companies or trusts on the IDS has not been supplied.

                Legacy Rule Format:
                    IF ([CTR156] > 0 OR [CTR158] > 0 OR [CTR159] > 0) AND ([IDS185] = NULL AND [IDS186] = NULL AND [IDS187] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR156] = CTR:RP.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [CTR158] = CTR:RP.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [CTR159] = CTR:RP:rvctc4.02.00:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount
                    [IDS185] = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS186] = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS187] = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count

            Technical Business Rule Format:
                    (^CTR156 > 0 OR ^CTR158 > 0 OR ^CTR159 > 0) AND (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL)

        Data Elements:
            
                    RP.CFCLC:^CTR156 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.CFCUC:^CTR158 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP:^CTR159 = tns:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount
            
                    RP:^IDS185 = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
            
                    RP:^IDS186 = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
            
                    RP:^IDS187 = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    */
        protected void VRATOCTR428416(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428416
            
            assertion = false; // = ((report.CTR156.GetValueOrDefault() > 0 || report.CTR158.GetValueOrDefault() > 0 || report.CTR159.GetValueOrDefault() > 0) && ( == null &&  == null &&  == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of controlled foreign companies or trusts on International Dealing Schedule is required",
                    LongDescription = @"Attributed foreign income has been completed on the company return form and the number of controlled foreign companies or trusts on the IDS has not been supplied.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428416"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR156", Value = report.CTR156.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR158", Value = report.CTR158.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR159", Value = report.CTR159.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428416

        #region VR.ATO.CTR.428417

        /*  VR.ATO.CTR.428417
                    If there is a yes response to question 'Did you have dealings with international related parties located in specified countries?' within the International Dealings Schedule (IDS), there must also be a yes response to 'Overseas transactions - Transactions with specified countries' on the company return form.

                Legacy Rule Format:
                    IF [CTR198] = FALSE AND [IDS9] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR198] = CTR:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator

            Technical Business Rule Format:
                    ^CTR198 = FALSE AND ^IDS9 = TRUE

        Data Elements:
            
                    RP:^CTR198 = tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            
                    RP:^IDS9 = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    */
        protected void VRATOCTR428417(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428417
            
            assertion = false; // = (report.CTR198 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = @"If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the International Dealings Schedule (IDS), then the response at 'Overseas transactions - Transactions with specified countries' on the company return must be 'True'.",
                    //Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428417"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR198", Value = GetValueOrEmpty(report.CTR198) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428417

        #region VR.ATO.CTR.428418

        /*  VR.ATO.CTR.428418
                    An amount is showing at question 5 Label C on IDS and there is no amount at cost of sales (Item 6 label A on the company return form.

                Legacy Rule Format:
                    IF [CTR73] = NULL AND ([IDS17] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR73] = CTR:RP:bafpr2.02.16:Expense.Operating.CostOfSales.Amount
                    [IDS17] = IDS:RP.TangibleRevenue:bafpr1.02.16:Expense.Total.Amount

            Technical Business Rule Format:
                    ^CTR73 = NULL AND (^IDS17 > 0)

        Data Elements:
            
                    RP:^CTR73 = tns:Expense.Operating.CostOfSales.Amount
            
                    RP:^IDS17 = IDS:RP.TangibleRevenue:bafpr1.02.16:Expense.Total.Amount
                    */
        protected void VRATOCTR428418(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428418
            
            assertion = false; // = (report.CTR73 == null &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428418", Severity = ProcessMessageSeverity.Error,
                    Description = @"Cost of sales required",
                    LongDescription = @"If an expenditure amount has been completed at 'Tangible property of a revenue nature including trading stock and raw materials' on the IDS then Cost of sales must be completed on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Expense.Operating.CostOfSales.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428418"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR73", Value = report.CTR73.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428418

        #region VR.ATO.CTR.428419

        /*  VR.ATO.CTR.428419
                    An amount is showing at question 5 Label D on IDS and there is no amount at Other sales of goods and services (Item 6 label C on the company return form

                Legacy Rule Format:
                    IF ([CTR59] = NULL OR [CTR59] = 0) AND [IDS18] > 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR59] = CTR:RP:bafpr2.02.02:Income.SaleOfGoodsAndServices.Other.Amount
                    [IDS18] = IDS:RP.TangibleRevenue:bafpr1.02.26:Revenue.Total.Amount

            Technical Business Rule Format:
                    (^CTR59 = NULL OR ^CTR59 = 0) AND ^IDS18 > 0

        Data Elements:
            
                    RP:^CTR59 = tns:Income.SaleOfGoodsAndServices.Other.Amount
            
                    RP:^IDS18 = IDS:RP.TangibleRevenue:bafpr1.02.26:Revenue.Total.Amount
                    */
        protected void VRATOCTR428419(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428419
            
            assertion = false; // = ((report.CTR59 == null || report.CTR59 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428419", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other sales of goods and services required",
                    LongDescription = @"If a revenue amount has been completed at 'Tangible property of a revenue nature including trading stock and raw materials' on the IDS then Other sales of goods and services must be greater than zero on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Income.SaleOfGoodsAndServices.Other.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428419"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR59", Value = report.CTR59.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428419

        #region VR.ATO.CTR.428420

        /*  VR.ATO.CTR.428420
                    An amount is showing at debt deductions in earning non-assessable non-exempt foreign income claimed under s25-90 or 230-15(3) and there is no amount at Interest expenses within Australia or Interest expenses overseas on the company return.

                Legacy Rule Format:
                    IF ([CTR80] = NULL OR [CTR80] = 0 ) AND ([CTR81] = NULL OR [CTR81] = 0 ) AND (COUNT(SCHEDULE = "IDS") = 1) AND ([IDS207] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR80] = CTR:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount
                    [CTR81] = CTR:RP.JFOREIGN:bafpr2.02.17:Expense.Operating.Interest.Amount
                    [IDS207] = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount

            Technical Business Rule Format:
                    (^CTR80 = NULL OR ^CTR80 = 0) AND (^CTR81 = NULL OR ^CTR81 = 0) AND (CountDocument('IDS') = 1) AND (^IDS207 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^CTR81 = tns:Expense.Operating.Interest.Amount
            
                    RP.JAUS:^CTR80 = tns:Expense.Operating.Interest.Amount
            
                    RP.JAUS:^IDS207 = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
                    */
        protected void VRATOCTR428420(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428420
            
            assertion = false; // = ((report.CTR80 == null || report.CTR80 == 0) && (report.CTR81 == null || report.CTR81 == 0) && @"IDS") == 1 &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430046", Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest expenses required",
                    LongDescription = @"If Debt deductions in earning non-assessable non-exempt foreign income claimed under s25-90 or 230-15(3) has been completed on the International dealings schedule (IDS) then Interest expenses within Australia or Interest expenses overseas must be greater than zero on the company return",
                    //Location = "/xbrli:xbrl/tns:Expense.Operating.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428420"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR80", Value = report.CTR80.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR81", Value = report.CTR81.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428420

        #region VR.ATO.CTR.428421

        /*  VR.ATO.CTR.428421
                    An amount is showing at Total amount of your share of net income/distribution of profit (question 29 Label C on IDS) and there is no amount at Item 7 other assessable income, Label B on the company return form.

                Legacy Rule Format:
                    IF ([CTR95] = NULL OR [CTR95] = 0) AND ([IDS219] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR95] = CTR:RP:bafpr1.02.02:Income.AssessableOther.Amount
                    [IDS219] = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount

            Technical Business Rule Format:
                    (^CTR95 = NULL OR ^CTR95 = 0) AND (^IDS219 > 0)

        Data Elements:
            
                    RP:^CTR95 = tns:Income.AssessableOther.Amount
            
                    RP:^IDS219 = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    */
        protected void VRATOCTR428421(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428421
            
            assertion = false; // = ((report.CTR95 == null || report.CTR95 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428421", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other assessable income required",
                    LongDescription = @"If Total amount of your share of net income/distribution of profit has been completed on the IDS then Other assessable income must be completed on the company return form.",
                    //Location = "/xbrli:xbrl/tns:Income.AssessableOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428421"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR95", Value = report.CTR95.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428421

        #region VR.ATO.CTR.428422

        /*  VR.ATO.CTR.428422
                    An amount is showing at question 26 on IDS and there is no amount at net capital gain (Item 7 label A on the company return form, Item 21 label A on the partnership and trust return forms).

                Legacy Rule Format:
                    IF (([CTR91] = NULL) AND ([IDS209] > 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR91] = CTR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [IDS209] = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount

            Technical Business Rule Format:
                    ((^CTR91 = NULL) AND (^IDS209 > 0))

        Data Elements:
            
                    RP:^CTR91 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^IDS209 = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
                    */
        protected void VRATOCTR428422(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428422
            
            assertion = false; // = (report.CTR91 == null &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428422", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain required",
                    LongDescription = @"If Capital gain amounts has been completed on the IDS then Net capital gain must be completed on the main return.",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428422"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR91", Value = report.CTR91.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428422

        #region VR.ATO.CTR.428424

        /*  VR.ATO.CTR.428424
                    There is a yes response to question 41 label A on the IDS, and there is no amount (label is blank) at Offshore banking unit adjustment, item 7 label P (label P can be zero) on the company return form.

                Legacy Rule Format:
                    IF ([CTR113] = NULL) AND ([IDS268] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR113] = CTR:RP:gfagc.02.00:TaxConcession.OffshoreBankingUnitAdjustment.Amount
                    [IDS268] = IDS:RP:bafot.02.11:FinancialServices.OffshoreBankingUnitOrHeadConsolidatedGroup.Indicator

            Technical Business Rule Format:
                    (^CTR113 = NULL) AND (^IDS268 = TRUE)

        Data Elements:
            
                    RP:^CTR113 = tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount
            
                    RP:^IDS268 = IDS:RP:bafot.02.11:FinancialServices.OffshoreBankingUnitOrHeadConsolidatedGroup.Indicator
                    */
        protected void VRATOCTR428424(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428424
            
            assertion = false; // = (report.CTR113 == null &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428424", Severity = ProcessMessageSeverity.Error,
                    Description = @"Offshore banking unit adjustment required",
                    LongDescription = @"If there is a 'True' response to 'Were you an offshore banking unit (OBU) or the head company of a consolidated group that included an OBU?' on the IDS then Offshore banking unit adjustment must be completed (zero is acceptable) on the company return form.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428424"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR113", Value = report.CTR113.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428424

        #region VR.ATO.CTR.428425

        /*  VR.ATO.CTR.428425
                    If an amount (including zero) is present at offshore banking unit adjustment, item 7 label P in the company return, then the response at question 41, label A must be 'yes' and the relevant labels at question 41a-41n need to be completed. The correct response is to be inserted at question 41 on the IDS or the amount(s) at item 7 label P on the company return is to be deleted.

                Legacy Rule Format:
                    IF ([CTR113] <> NULL) AND ([IDS268] = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR113] = CTR:RP:gfagc.02.00:TaxConcession.OffshoreBankingUnitAdjustment.Amount
                    [IDS268] = IDS:RP:bafot.02.11:FinancialServices.OffshoreBankingUnitOrHeadConsolidatedGroup.Indicator

            Technical Business Rule Format:
                    (^CTR113 <> NULL) AND (^IDS268 = FALSE)

        Data Elements:
            
                    RP:^CTR113 = tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount
            
                    RP:^IDS268 = IDS:RP:bafot.02.11:FinancialServices.OffshoreBankingUnitOrHeadConsolidatedGroup.Indicator
                    */
        protected void VRATOCTR428425(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428425
            
            assertion = false; // = (report.CTR113 != null &&  == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428425", Severity = ProcessMessageSeverity.Error,
                    Description = @"Incorrect response for an OBU",
                    LongDescription = @"If Offshore banking unit adjustment has been completed (including zero) on the company return form then there must be a 'True' response at 'Were you an offshore banking unit (OBU) or the head company of a consolidated group that included an OBU?' on the IDS, and the relevant amount must be completed.",
                    //Location = "/xbrli:xbrl/tns:TaxConcession.OffshoreBankingUnitAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428425"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR113", Value = report.CTR113.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428425

        #region VR.ATO.CTR.428426

        /*  VR.ATO.CTR.428426
                    If the response at "Has an Australian resident company elected under subdivision 820-FB of the ITAA 1997 to treat your qualifying Australian branch operations as part of a consolidated group, MEC group or a single company for thin capitalisation purposes?" is 'true' , then the status of the company at Non-resident with permanent establishment on the company return.

                Legacy Rule Format:
                    IF ([CTR51] <> "NonResidentPermEstab") AND ([IDS221] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR51] = CTR:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
                    [IDS221] = IDS:RP:bafpo6.02.09:Liabilities.ThinCapitalisation.AustralianResidentCompanyElected.Indicator

            Technical Business Rule Format:
                    (^CTR51 <> 'NonResidentPermEstab') AND (^IDS221 = TRUE)

        Data Elements:
            
                    RP:^CTR51 = tns:Residency.TaxPurposesOrganisationStatus.Code
            
                    RP:^IDS221 = IDS:RP:bafpo6.02.09:Liabilities.ThinCapitalisation.AustralianResidentCompanyElected.Indicator
                    */
        protected void VRATOCTR428426(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428426
            
            assertion = false; // = (report.CTR51 != @"NonResidentPermEstab" &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Status of company is incorrect",
                    LongDescription = @"If there is a 'True' response to ""Has an Australian resident company elected under subdivision 820-FB of the ITAA 1997 to treat your qualifying Australian branch operations as part of a consolidated group, MEC group or a single company for thin capitalisation purposes?"" on the IDS then the residency status at Status of company must be Non-resident with permanent establishment on the company return",
                    //Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428426"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR51", Value = report.CTR51 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428426

        #region VR.ATO.CTR.428427

        /*  VR.ATO.CTR.428427
                    Amounts are showing at question 26 on IDS but there is no 'yes' response to the question concerning whether you have a CGT event during the year (Item 7 label G on the company return form; item 21 label G on the partnership and trust return forms).

                Legacy Rule Format:
                    IF ([CTR89] = FALSE) AND ([IDS208] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR89] = CTR:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
                    [IDS208] = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator

            Technical Business Rule Format:
                    (^CTR89 = FALSE) AND (^IDS208 = TRUE)

        Data Elements:
            
                    RP:^CTR89 = tns:CapitalGainsTax.Event.Indicator
            
                    RP:^IDS208 = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
                    */
        protected void VRATOCTR428427(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.428427
            
            assertion = false; // = (report.CTR89 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.428427", Severity = ProcessMessageSeverity.Error,
                    Description = @"Check response to CGT event question",
                    LongDescription = @"If there is a 'True' response to 'Did you have any capital gains tax (CGT) events in relation to your interest in a foreign company?' On the IDS then a 'True' response is required at 'Did you have a CGT event during the year?' on the company return.",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.Event.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.428427"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR89", Value = GetValueOrEmpty(report.CTR89) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.428427

        #region VR.ATO.CTR.430014

        /*  VR.ATO.CTR.430014
                    International dealings schedule must be attached if interest expenses overseas amount is greater than zero

                Legacy Rule Format:
                    IF [CTR81] > 0 AND (COUNT(SCHEDULE = "IDS") = 0) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR81] = CTR:RP.JFOREIGN:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
                    ^CTR81 > 0 AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^CTR81 = tns:Expense.Operating.Interest.Amount
                    */
        protected void VRATOCTR430014(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.430014
            
            assertion = false; // = (report.CTR81.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430014", Severity = ProcessMessageSeverity.Error,
                    Description = @"International dealings schedule required",
                    LongDescription = @"International dealings schedule must be attached if interest expenses overseas amount is greater than zero",
                    //Location = "/xbrli:xbrl/tns:Expense.Operating.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.430014"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR81", Value = report.CTR81.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.430014

        #region VR.ATO.CTR.430015

        /*  VR.ATO.CTR.430015
                    International dealings schedule must be attached if royalty expenses overseas amount is greater than zero

                Legacy Rule Format:
                    IF [CTR366] > 0 AND (COUNT(SCHEDULE = "IDS") = 0) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR366] = CTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    ^CTR366 > 0 AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^CTR366 = tns:Expense.Royalties.Amount
                    */
        protected void VRATOCTR430015(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.430015
            
            assertion = false; // = (report.CTR366.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430015", Severity = ProcessMessageSeverity.Error,
                    Description = @"International dealings schedule required",
                    LongDescription = @"International dealings schedule must be attached if royalty expenses overseas amount is greater than zero",
                    //Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.430015"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR366", Value = report.CTR366.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.430015

        #region VR.ATO.CTR.430020

        /*  VR.ATO.CTR.430020
                    International dealings schedule must be attached if section 46FA deduction for flow-on dividends amount is greater than zero

                Legacy Rule Format:
                    IF [CTR98] > 0 AND (COUNT(SCHEDULE = "IDS") = 0) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR98] = CTR:RP:bafpr1.02.02:Expense.FlowOnDividendDeduction.Amount

            Technical Business Rule Format:
                    ^CTR98 > 0 AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP:^CTR98 = tns:Expense.FlowOnDividendDeduction.Amount
                    */
        protected void VRATOCTR430020(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.430020
            
            assertion = false; // = (report.CTR98.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430020", Severity = ProcessMessageSeverity.Error,
                    Description = @"International dealings schedule required",
                    LongDescription = @"International dealings schedule must be attached if section 46FA deduction for flow-on dividends amount is greater than zero",
                    //Location = "/xbrli:xbrl/tns:Expense.FlowOnDividendDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.430020"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR98", Value = report.CTR98.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.430020

        #region VR.ATO.CTR.430023

        /*  VR.ATO.CTR.430023
                    The net capital losses carried forward (including collectables) must be consistent between the income tax return and the CGT schedule

                Legacy Rule Format:
                    IF [CTR176] <> NULL  AND (COUNT(SCHEDULE = "CGTS") = 1) AND ([CTR176] <> ([CGTS124] + [CGTS125]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CTR176] = CTR:RP(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    ^CTR176 <> NULL  AND (CountDocument('CGTS') = 1) AND (^CTR176 <> (^CGTS124 + ^CGTS125))

        Data Elements:
            
                    RP(Instant):^CTR176 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP(Instant):^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            
                    RP(Instant):^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOCTR430023(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.430023
            
            assertion = false; // = (report.CTR176 != null && @"CGTS") == 1 && report.CTR176.GetValueOrDefault() !=  + );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years incorrect",
                    LongDescription = @"Net capital losses carried forward to later income years on the company return  must equal the total net capital losses carried forward (including collectables) to later income years on the CGT schedule",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.430023"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR176", Value = report.CTR176.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.430023

        #region VR.ATO.CTR.430031

        /*  VR.ATO.CTR.430031
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Intercompany rollover Subdivision 126B", then Capital gains disregarded as a result of inter-company assets roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF ([CTR330] = "G - Inter-company roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS46] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS46] = CGTS:RP:rvctc3.02.11:Capital.Gains.InterCompanyAssetRolloverDeferral.Amount
                    [CTR330] = CTR:RP:rvctc3.02.33:CapitalGainsTax.ExemptionOrRolloverApplied.Code

            Technical Business Rule Format:
                    (^CTR330 = 'G - Inter-company roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS46 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS46 = CGTS:RP:rvctc3.02.11:Capital.Gains.InterCompanyAssetRolloverDeferral.Amount
                    */
        protected void VRATOCTR430031(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.430031
            
            assertion = false; // = (CTR330 == @"G - Inter-company roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.430031", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of inter-company assets roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Intercompany rollover Subdivision 126B"", then Capital gains disregarded as a result of inter-company assets roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.430031"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.430031

        #region VR.ATO.CTR.438050

        /*  VR.ATO.CTR.438050
                    If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS

                Legacy Rule Format:
                    IF ([CTR310] > 0 OR [CTR311] > 0) 
                    AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS177] <> TRUE)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CTR310] = CTR:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount
                    [CTR311] = CTR:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount
                    [IDS177] = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator

            Technical Business Rule Format:
                    (^CTR310 > 0 OR ^CTR311 > 0) AND (CountDocument('IDS') = 1 AND ^IDS177 <> TRUE)

        Data Elements:
            
                    RP.TOFA:^CTR310 = tns:Income.GainsTotal.Amount
            
                    RP.TOFA:^CTR311 = tns:Expense.LossesTotal.Amount
            
                    RP.TOFA:^IDS177 = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
                    */
        protected void VRATOCTR438050(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.CTR.438050
            
            assertion = false; // = ((report.CTR310.GetValueOrDefault() > 0 || report.CTR311.GetValueOrDefault() > 0) && (@"IDS") == 1 &&  != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.438050", Severity = ProcessMessageSeverity.Error,
                    Description = @"Response to 'Are you subject to the TOFA rules contained in Division 230 of the ITAA 1997' is incorrect",
                    LongDescription = @"If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS",
                    //Location = "/xbrli:xbrl/tns:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.CTR.438050"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR310", Value = report.CTR310.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR311", Value = report.CTR311.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.438050

        #region VR.ATO.GEN.438022

        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'A - Small business active asset reduction') AND (CountDocument('CGTS') = 1) AND ^CGTS127 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS127 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                    */
        protected void VRATOGEN438022(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438022
            
            assertion = false; // = (CTR330 == @"A - Small business active asset reduction" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438022", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022

        #region VR.ATO.GEN.438023

        /*  VR.ATO.GEN.438023
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS128] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'B - Small business retirement exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS128 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS128 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
                    */
        protected void VRATOGEN438023(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438023
            
            assertion = false; // = (CTR330 == @"B - Small business retirement exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438023

        #region VR.ATO.GEN.438024

        /*  VR.ATO.GEN.438024
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'C - Small business roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS129 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS129 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                    */
        protected void VRATOGEN438024(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438024
            
            assertion = false; // = (CTR330 == @"C - Small business roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438024", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024

        #region VR.ATO.GEN.438025

        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                    */
        protected void VRATOGEN438025(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438025
            
            assertion = false; // = (CTR330 == @"D - Small business 15 year exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438025", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025

        #region VR.ATO.GEN.438026

        /*  VR.ATO.GEN.438026
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS140] = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'E - Foreign resident CGT exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS140 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS140 = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount
                    */
        protected void VRATOGEN438026(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438026
            
            assertion = false; // = (CTR330 == @"E - Foreign resident CGT exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438026", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438026

        #region VR.ATO.GEN.438027

        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS45 = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                    */
        protected void VRATOGEN438027(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438027
            
            assertion = false; // = (CTR330 == @"F - Scrip for scrip roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438027", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027

        #region VR.ATO.GEN.438028

        /*  VR.ATO.GEN.438028
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Demerger exemption (Subdivision 125-C)", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "H - Demerger exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS47] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS47] = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount

            Technical Business Rule Format:
                    (^CTR330 = 'H - Demerger exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS47 = NULL

        Data Elements:
            
                    RP:^CTR330 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS47 = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount
                    */
        protected void VRATOGEN438028(List<ProcessMessageDocument> response, CTR2017 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438028
            
            assertion = false; // = (CTR330 == @"H - Demerger exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438028", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a demerging entity amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Demerger exemption (Subdivision 125-C)"", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438028"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CTR330", Value = CTR330 });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438028
    #endregion Cross Form Rules - not supported yet
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(CTR2017 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOCTR428027(response, report);
            VRATOCTR428034(response, report);
            VRATOCTR428037(response, report);
            VRATOCTR428039(response, report);
            VRATOCTR428040(response, report);
            VRATOCTR428096(response, report);
            VRATOCTR428098(response, report);
            VRATOCTR428099(response, report);
            VRATOCTR428100(response, report);
            VRATOCTR428101(response, report);
            VRATOCTR428102(response, report);
            VRATOCTR428109(response, report);
            VRATOCTR428135(response, report);
            VRATOCTR428187(response, report);
            VRATOCTR428190(response, report);
            VRATOCTR428191(response, report);
            VRATOCTR428234(response, report);
            VRATOCTR428240(response, report);
            VRATOCTR428278(response, report);
            VRATOCTR428352(response, report);
            VRATOCTR428353(response, report);
            VRATOCTR428354(response, report);
            VRATOCTR428355(response, report);
            VRATOCTR428398(response, report);
            VRATOCTR428399(response, report);
            VRATOCTR428400(response, report);
            VRATOCTR428404(response, report);
            VRATOCTR428405(response, report);
            VRATOCTR428406(response, report);
            VRATOCTR428408(response, report);
            VRATOCTR428409(response, report);
            VRATOCTR428410(response, report);
            VRATOCTR428412(response, report);
            VRATOCTR428413(response, report);
            VRATOCTR428414(response, report);
            VRATOCTR428415(response, report);
            VRATOCTR428416(response, report);
            VRATOCTR428417(response, report);
            VRATOCTR428418(response, report);
            VRATOCTR428419(response, report);
            VRATOCTR428420(response, report);
            VRATOCTR428421(response, report);
            VRATOCTR428422(response, report);
            VRATOCTR428424(response, report);
            VRATOCTR428425(response, report);
            VRATOCTR428426(response, report);
            VRATOCTR428427(response, report);
            VRATOCTR430014(response, report);
            VRATOCTR430015(response, report);
            VRATOCTR430020(response, report);
            VRATOCTR430023(response, report);
            VRATOCTR430031(response, report);
            VRATOCTR438050(response, report);
            VRATOGEN438022(response, report);
            VRATOGEN438023(response, report);
            VRATOGEN438024(response, report);
            VRATOGEN438025(response, report);
            VRATOGEN438026(response, report);
            VRATOGEN438027(response, report);
            VRATOGEN438028(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
