using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;
using System.IO;
using System.Text;
using System.Xml.Schema;
using System.Reflection;
using System.Runtime.Serialization; 

namespace Ato.EN.IntegrationServices.CodeGenerationPTR
{
    public partial class PTR2017Validator
    {

        #region Cross Form Rules - not supported yet

        #region VR.ATO.GEN.430008

        /*  VR.ATO.GEN.430008
                    An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "IEE") > 0) AND RP:entity.identifier.TFN = (ANY OCCURRENCE OF([IEE50])) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.xx.xx:FamilyTrust)

            Technical Business Rule Format:
                    (CountDocument('IEE') > 0) AND AnyOccurrence(^IEE50, ^IEE50 = ^TFN)

        Data Elements:
            
                    ^IEE50 = IEE:RP:pyid.66.66:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.66.66:FamilyTrust)
            
                    ^TFN = RP
                    */
        protected void VRATOGEN430008(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.430008

            assertion = false; // = (@"IEE") > 0 && ( == null ? false : .Any(IEE50Repeat =>  == report.RPIdentifierTFN)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430008" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 

                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.430008

        #region VR.ATO.GEN.438002

        /*  VR.ATO.GEN.438002
                    If the response to the question to whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust (Attributed foreign income) is 'Yes, an International Dealings Schedule must be attached

                Legacy Rule Format:
                    IF (bafpr3.xx.xx:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^PTR102 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^PTR102 = bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
                    */
        protected void VRATOGEN438002(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438002

            assertion = false; // = (report.PTR102 == true && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not attached",
                    LongDescription = @"The answer to the question 'Did the entity have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?' is 'Yes', and no International Dealings Schedule is attached.",
                    //Location = "/xbrli:xbrl/bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438002" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR102", Value = GetValueOrEmpty(report.PTR102) });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438002

        #region VR.ATO.GEN.438003

        /*  VR.ATO.GEN.438003
                    If the response to the question to whether the aggregate amount of the transactions or dealings with international related parties (including the value of any property or service transferred or the balance of any loans) greater than $2 million? (Overseas transactions) is 'Yes', an International Dealings Schedule must be attached

                Legacy Rule Format:
                    IF (rvctc4.xx.xx:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^PTR108 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP:^PTR108 = rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    */
        protected void VRATOGEN438003(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438003

            assertion = false; // = (report.PTR108 == true && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not attached",
                    LongDescription = @"The answer to the question 'Was the aggregate amount of your transactions or dealings with international related parties (including the value of any property or service transferred or the balance of any loans) greater than $2 million?'(Overseas transactions) is 'Yes' , and no International Dealings Schedule is attached.",
                    //Location = "/xbrli:xbrl/rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438003" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR108", Value = GetValueOrEmpty(report.PTR108) });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438003

        #region VR.ATO.GEN.438017

        /*  VR.ATO.GEN.438017
                    If the thin capitalisation rules apply  then International Dealings Schedule must be attached

                Legacy Rule Format:
                    IF (RP:bafpo6.xx.xx:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0)
                      RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^PTR277 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP:^PTR277 = bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOGEN438017(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438017

            assertion = false; // = (report.PTR277 == true && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"If the thin capitalisation rules apply  then International Dealings Schedule must be attached",
                    //Location = "/xbrli:xbrl/bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438017" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR277", Value = GetValueOrEmpty(report.PTR277) });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438017

        #region VR.ATO.PTR.430019

        /*  VR.ATO.PTR.430019
                    Payment Summary schedule must be present if Gross payments where ABN not quoted is present

                Legacy Rule Format:
                    IF [PTR35] + [PTR36] > 0 AND (COUNT(SCHEDULE = "PSS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    ^PTR35 + ^PTR36 > 0 AND (CountDocument('PSS') = 0)

        Data Elements:
            
                    RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOPTR430019(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430019

            assertion = false; // = (report.PTR35.GetValueOrDefault() + report.PTR36.GetValueOrDefault() > 0 && @"PSS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430019",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must be present if Gross payments where ABN not quoted is present",
                    //Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430019" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430019

        #region VR.ATO.PTR.430020

        /*  VR.ATO.PTR.430020
                    Gross payment where ABN not quoted must equal sum of payments on schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") > 0)  AND SUM([PSS19]) <> [PTR35] + [PTR36] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0)  AND Sum(^PSS19) <> ^PTR35 + ^PTR36

        Data Elements:
            
                    RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.Prim:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOPTR430020(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430020

            assertion = false; // = (@"PSS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.PTR35.GetValueOrDefault() + report.PTR36.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430020",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment where ABN not quoted must equal sum of payments on schedule",
                    LongDescription = @"The sum of gross payment with indicator 'N' in the Payment Summary schedule must equal the sum of Primary production and Non-primary production Gross payments where ABN not quoted on main return form",
                    //Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430020" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430020

        #region VR.ATO.PTR.430025

        /*  VR.ATO.PTR.430025
                    Gross payment - Foreign Resident Withholding (excluding capital gains) is incorrect

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") > 0) AND [PTR38] <> SUM([PSS18])  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0) AND ^PTR38 <> Sum(^PSS18)

        Data Elements:
            
                    RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP.JAUS.NonPrim:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    */
        protected void VRATOPTR430025(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430025

            assertion = false; // = (@"PSS") > 0 && report.PTR38.GetValueOrDefault() != ( == null ? 0 : .Sum(f => f.)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment - Foreign Resident Withholding (excluding capital gains) is incorrect",
                    LongDescription = @"The sum of gross payments with indicator 'F' in the Payment Summary schedule must be equal to the sum of total gross payment subject to foreign resident withholding (excluding capital gains) primary production and total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form",
                    //Location = "/xbrli:xbrl/lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430025" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430025

        #region VR.ATO.PTR.430081

        /*  VR.ATO.PTR.430081
                    Rental schedule must be present

                Legacy Rule Format:
                    IF ([PTR77] <> NULL OR [PTR78] > 0 OR [PTR79] > 0 OR [PTR80] > 0) AND (COUNT(SCHEDULE = "RS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (^PTR77 <> NULL OR ^PTR78 > 0 OR ^PTR79 > 0 OR ^PTR80 > 0) AND (CountDocument('RS') = 0)

        Data Elements:
            
                    RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
            
                    RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    */
        protected void VRATOPTR430081(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430081

            assertion = false; // = ((report.PTR77 != null || report.PTR78.GetValueOrDefault() > 0 || report.PTR79.GetValueOrDefault() > 0 || report.PTR80.GetValueOrDefault() > 0) && @"RS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental schedule must be present",
                    LongDescription = @"If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property",
                    //Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430081" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430081

        #region VR.ATO.PTR.430082

        /*  VR.ATO.PTR.430082
                    Gross rent and Rental deduction amounts must be present

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND ([PTR77] = NULL AND ([PTR78] = 0 OR [PTR78] = NULL) AND ([PTR79] = 0 OR [PTR79] = NULL) AND ([PTR80] = 0 OR [PTR80] = NULL)) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (^PTR77 = NULL AND (^PTR78 = 0 OR ^PTR78 = NULL) AND (^PTR79 = 0 OR ^PTR79 = NULL) AND (^PTR80 = 0 OR ^PTR80 = NULL))

        Data Elements:
            
                    RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
            
                    RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    */
        protected void VRATOPTR430082(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430082

            assertion = false; // = (@"RS") > 0 && (report.PTR77 == null && (report.PTR78 == 0 || report.PTR78 == null) && (report.PTR79 == 0 || report.PTR79 == null) && (report.PTR80 == 0 || report.PTR80 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and Rental deduction amounts must be present",
                    LongDescription = @"If a Rental property schedule is present, Gross rent or rental deductions must be present",
                    //Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430082" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430082

        #region VR.ATO.PTR.430083

        /*  VR.ATO.PTR.430083
                    Gross rent must equal the total Gross rent in the Rental Schedules

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS38] + [RS39])) <> [PTR77])
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [RS38] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
                    [RS39] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS38 + ^RS39)) <> ^PTR77)

        Data Elements:
            
                    RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^RS38 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
            
                    RP.JAUS:^RS39 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount
                    */
        protected void VRATOPTR430083(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430083

            assertion = false; // = (@"RS") > 0 &&  + ) != report.PTR77.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must equal the total Gross rent in the Rental Schedules",
                    LongDescription = @"The Gross rent amount in the main form must equal the total Gross rent present in all Rental property schedules",
                    //Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430083" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430083

        #region VR.ATO.PTR.430084

        /*  VR.ATO.PTR.430084
                    Net rent must equal the total net rent in the Rental Schedules

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND SUM(ALL OCCURRENCES OF([RS59])) <> ([PTR77] - [PTR78] - [PTR79] - [PTR80]) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    [RS59] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS59) <> (^PTR77 - ^PTR78 - ^PTR79 - ^PTR80)

        Data Elements:
            
                    RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
            
                    RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            
                    RP.JAUS:^RS59 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount
                    */
        protected void VRATOPTR430084(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430084

            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.PTR77.GetValueOrDefault() - report.PTR78.GetValueOrDefault() - report.PTR79.GetValueOrDefault() - report.PTR80.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rent must equal the total net rent in the Rental Schedules",
                    LongDescription = @"The net rent amount in the main form (Gross rent minus rental deduction amounts) must equal the total Net rent present in all Rental property schedules",
                    //Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430084" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430084

        #region VR.ATO.PTR.430086

        /*  VR.ATO.PTR.430086
                    Rental Interest deductions must equal the total Interest deductions in the Rental Schedules

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS48]))) <> [PTR78] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [RS48] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum(^RS48)) <> ^PTR78

        Data Elements:
            
                    RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
            
                    RP.JAUS:^RS48 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount
                    */
        protected void VRATOPTR430086(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430086

            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.PTR78.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental Interest deductions must equal the total Interest deductions in the Rental Schedules",
                    LongDescription = @"The Interest deductions amount in the main form must equal the total of all Interest deductions in the Rental property schedules",
                    //Location = "/xbrli:xbrl/bafpr1.02.05:Expense.Interest.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430086" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430086

        #region VR.ATO.PTR.430088

        /*  VR.ATO.PTR.430088
                    Capital works deductions must equal the total Capital works deductions in the Rental Schedules

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS54]))) <> [PTR79]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [RS54] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum(^RS54)) <> ^PTR79

        Data Elements:
            
                    RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^RS54 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                    */
        protected void VRATOPTR430088(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430088

            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.PTR79.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital works deductions must equal the total Capital works deductions in the Rental Schedules",
                    LongDescription = @"The Capital works deductions amount in the main form must equal the total of all Capital works deductions in the Rental property schedules",
                    //Location = "/xbrli:xbrl/bafpr1.02.02:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430088" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430088

        #region VR.ATO.PTR.430090

        /*  VR.ATO.PTR.430090
                    Other rental deductions must equal the total other deductions in the Rental Schedules

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS40] + [RS41] + [RS42] + [RS43] + [RS44] + [RS45] + [RS46] + [RS47] + [RS49] + [RS50] + [RS51] + [RS52] + [RS53] + [RS55] + [RS56] + [RS57] + [RS58])) <> [PTR80])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    [RS40] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
                    [RS41] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
                    [RS42] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
                    [RS43] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
                    [RS44] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
                    [RS45] = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
                    [RS46] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
                    [RS47] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
                    [RS49] = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
                    [RS50] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
                    [RS51] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
                    [RS52] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
                    [RS53] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                    [RS55] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
                    [RS56] = RS:RP:bafpr1.02.00:Expense.Travel.Amount
                    [RS57] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
                    [RS58] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58)) <> ^PTR80)

        Data Elements:
            
                    RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            
                    RP.JAUS:^RS40 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
            
                    RP.JAUS:^RS41 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
            
                    RP.JAUS:^RS42 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
            
                    RP.JAUS:^RS43 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
            
                    RP.JAUS:^RS44 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
            
                    RP.JAUS:^RS45 = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
            
                    RP.JAUS:^RS46 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
            
                    RP.JAUS:^RS47 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
            
                    RP.JAUS:^RS49 = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
            
                    RP.JAUS:^RS50 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
            
                    RP.JAUS:^RS51 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
            
                    RP.JAUS:^RS52 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
            
                    RP.JAUS:^RS53 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
            
                    RP.JAUS:^RS55 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
            
                    RP.JAUS:^RS56 = RS:RP:bafpr1.02.00:Expense.Travel.Amount
            
                    RP.JAUS:^RS57 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
            
                    RP.JAUS:^RS58 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount
                    */
        protected void VRATOPTR430090(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430090

            assertion = false; // = (@"RS") > 0 &&  +  +  +  +  +  +  +  +  +  +  +  +  +  +  +  + ) != report.PTR80.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other rental deductions must equal the total other deductions in the Rental Schedules",
                    LongDescription = @"The Other rental deductions amount in the main form must equal the total of all rental expenses in the Rental property, excluding Interest deductions and Capital works deductions",
                    //Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430090" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430090

        #region VR.ATO.PTR.430176

        /*  VR.ATO.PTR.430176
                    Year of revocation must be the same as the year of the return

                Legacy Rule Format:
                    IF (PTR:RP:pyin.xx.xx:Report.TargetFinancial.Year <> NULL) AND (COUNT(SCHEDULE = "IEE") > 0) AND (ANY OCCURRENCE OF ([IEE116] = "R") AND ([IEE82] <> PTR:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE82] = IEE:RP:rvctc3.xx.xx:Elections.Revocation.Year
                    [IEE116] = IEE:RP:rvctc3.xx.xx:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
                    (^PTR1 <> NULL) AND (CountDocument('IEE') > 0) AND ((AnyOccurrence(^IEE116, ^IEE116 = 'R')) AND (^IEE82 <> ^PTR1))

        Data Elements:
            
                    RP:^PTR1 = pyin.02.37:Report.TargetFinancial.Year
            
                    RP:^IEE82 = IEE:RP:rvctc3.66.66:Elections.Revocation.Year
            
                    RP:^IEE116 = IEE:RP:rvctc3.66.66:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        protected void VRATOPTR430176(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430176

            assertion = false; // = (report.PTR1 != null && @"IEE") > 0 && (( == null ? false : .Any(IEE116Repeat =>  == @"R")) &&  != report.PTR1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of revocation must be the same as the year of the return",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    //Location = "/xbrli:xbrl/pyin.02.37:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430176" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR1", Value = report.PTR1.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430176

        #region VR.ATO.PTR.430178

        /*  VR.ATO.PTR.430178
                    IEE specified income year must be present

                Legacy Rule Format:
                    IF [PTR26] = NULL AND (COUNT(SCHEDULE = "IEE") > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR26] = PTR:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
                    ^PTR26 = NULL AND (CountDocument('IEE') > 0)

        Data Elements:
            
                    RP:^PTR26 = rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
                    */
        protected void VRATOPTR430178(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430178

            assertion = false; // = (report.PTR26 == null && @"IEE") > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year must be present",
                    LongDescription = @"If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    //Location = "/xbrli:xbrl/rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430178" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR26", Value = report.PTR26.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430178

        #region VR.ATO.PTR.430180

        /*  VR.ATO.PTR.430180
                    If the Interposed Entity Election revocation code is present, an IEE schedule must be present

                Legacy Rule Format:
                    IF ([PTR27] = "R") AND (COUNT(SCHEDULE = "IEE") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR27] = PTR:RP:rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code

            Technical Business Rule Format:
                    (^PTR27 = 'R') AND (CountDocument('IEE') = 0)

        Data Elements:
            
                    RP:^PTR27 = rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code
                    */
        protected void VRATOPTR430180(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430180

            assertion = false; // = (report.PTR27 == @"R" && @"IEE") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430180",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the Interposed Entity Election revocation code is present, an IEE schedule must be present",
                    //Location = "/xbrli:xbrl/rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430180" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR27", Value = report.PTR27 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430180

        #region VR.ATO.PTR.430183

        /*  VR.ATO.PTR.430183
                    Payment Summary schedule must include Tax withheld where ABN not quoted amount

                Legacy Rule Format:
                    IF ([PTR63] > 0) AND ((COUNT(SCHEDULE = "PSS") = 0) OR (SUM([PSS23]) <> [PTR63]))
                       RETURN VALIDATION ERROR
                    ENDIF
                    
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
                    (^PTR63 > 0) AND ((CountDocument('PSS') = 0) OR (Sum(^PSS23) <> ^PTR63))

        Data Elements:
            
                    RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    RP.JAUS:^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    */
        protected void VRATOPTR430183(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430183

            assertion = false; // = (report.PTR63.GetValueOrDefault() > 0 && (@"PSS") == 0 || ( == null ? 0 : .Sum(f => f.)) != report.PTR63.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430183",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must include Tax withheld where ABN not quoted amount",
                    LongDescription = @"If an amount greater than zero has been entered at Tax withheld from payments where ABN no quoted, then a Payment Summary schedule must be present and it must include the same total amount of Tax withheld from payments where ABN no quoted",
                    //Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430183" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430183

        #region VR.ATO.PTR.430995

        /*  VR.ATO.PTR.430995
                    If there is an amount at 'Did you have any international related party dealings involving royalties or licence fees?', on the IDS it must be less than or equal to the amount at 'Royalty expenses overseas'.

                Legacy Rule Format:
                    IF ([PTR138] = 0 OR [PTR138] = NULL) AND ([IDS22] > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^PTR138 = 0 OR ^PTR138 = NULL) AND (^IDS22 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOPTR430995(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430995

            assertion = false; // = ((report.PTR138 == 0 || report.PTR138 == null) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430995",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty expenses overseas required",
                    LongDescription = @"If a Royalties deduction amount has been completed on the IDS then Royalty expenses overseas must be greater than zero on the partnership form.",
                    //Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430995" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430995

        #region VR.ATO.PTR.430996

        /*  VR.ATO.PTR.430996
                    Where there is a Royalties deduction amount present on the IDS, there must also be an amount at Royalty expenses overseas on the partnership or trust return form.

                Legacy Rule Format:
                    IF ([PTR138] >  0) AND ([PTR138] < [IDS22])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^PTR138 >  0) AND (^PTR138 < ^IDS22)

        Data Elements:
            
                    RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOPTR430996(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430996

            assertion = false; // = (report.PTR138.GetValueOrDefault() > 0 && report.PTR138.GetValueOrDefault() < );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430996",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalties deductions incorrect",
                    LongDescription = @"If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the partnership return form.",
                    //Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430996" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430996

        #region VR.ATO.PTR.430998

        /*  VR.ATO.PTR.430998
                    The sum of the amounts showing at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or less than the sum of the amounts at Listed country and Unlisted country on the main return form.

                Legacy Rule Format:
                    IF ([PTR98] > 0 OR [PTR100] > 0) AND (([PTR98] +  [PTR100]) < ([IDS191] + [IDS192] + [IDS193]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS191] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
                    [IDS192] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
                    [IDS193] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
                    [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [PTR100] = PTR:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^PTR98 > 0 OR ^PTR100 > 0) AND ((^PTR98 +  ^PTR100) < (^IDS191 + ^IDS192 + ^IDS193))

        Data Elements:
            
                    RP.JFOREIGN.CFCUC:^PTR100 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS191 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS192 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS193 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
            
                    RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    */
        protected void VRATOPTR430998(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430998

            assertion = false; // = ((report.PTR98.GetValueOrDefault() > 0 || report.PTR100.GetValueOrDefault() > 0) && report.PTR98.GetValueOrDefault() + report.PTR100.GetValueOrDefault() <  +  + );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430998",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or less than the sum of amounts at Listed country and Unlisted country on the partnership return form.",
                    //Location = "/xbrli:xbrl/bafpr3.02.02:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430998" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR100", Value = report.PTR100.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430998

        #region VR.ATO.PTR.430999

        /*  VR.ATO.PTR.430999
                    The amount showing at section 456 –attributable income controlled foreign companies on the IDS should be equal to or less than the amount at Listed country.

                Legacy Rule Format:
                    IF ([PTR98] > 0) AND ([IDS188] > [PTR98])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS188] = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^PTR98 > 0) AND (^IDS188 > ^PTR98)

        Data Elements:
            
                    RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCLC:^IDS188 = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    */
        protected void VRATOPTR430999(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.430999

            assertion = false; // = (report.PTR98.GetValueOrDefault() > 0 &&  > report.PTR98.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430999",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"Assessable income of controlled foreign companies and controlled foreign trusts under section 456 for Listed countries on the IDS should be equal to or less than Attributed foreign income - Listed country on the partnership return.",
                    //Location = "/xbrli:xbrl/bafpr3.02.02:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430999" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.430999

        #region VR.ATO.PTR.431000

        /*  VR.ATO.PTR.431000
                    Listed country, Unlisted country or Transferor trust has been completed and the number of controlled foreign companies or trusts on the IDS, has not been shown. Show zero at 'Number of CFCs and CFTs - Listed countries', 'Number of CFCs and CFTs - Specified countries', 'Number of CFC and CFTs - Other unlisted countries' on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.

                Legacy Rule Format:
                    IF ([PTR98] > 0 OR [PTR100] > 0) AND ([IDS185] = NULL AND [IDS186] = NULL AND [IDS187] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS185] = IDS:RP.CFCLC:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
                    [IDS186] = IDS:RP.CFCSP:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
                    [IDS187] = IDS:RP.CFCUC:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
                    [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [PTR100] = PTR:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^PTR98 > 0 OR ^PTR100 > 0) AND (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL)

        Data Elements:
            
                    RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCLC:^IDS185 = IDS:RP.CFCLC:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
            
                    RP.JFOREIGN.CFCLC:^IDS186 = IDS:RP.CFCSP:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
            
                    RP.JFOREIGN.CFCLC:^IDS187 = IDS:RP.CFCUC:bafot.02.11:Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Count
            
                    RP.JFOREIGN.CFCUC:^PTR100 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    */
        protected void VRATOPTR431000(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431000

            assertion = false; // = ((report.PTR98.GetValueOrDefault() > 0 || report.PTR100.GetValueOrDefault() > 0) && ( == null &&  == null &&  == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431000",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of controlled foreign companies or trusts required",
                    LongDescription = @"Listed country, Unlisted country or Transferor trust has been completed and the number of controlled foreign companies or trusts on the IDS has not been shown. Show zero on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.",
                    //Location = "/xbrli:xbrl/bafpr3.02.02:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431000" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR100", Value = report.PTR100.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431000

        #region VR.ATO.PTR.431001

        /*  VR.ATO.PTR.431001
                    If there is a yes response to question 4 label A on the IDS, there must also be a yes response to question 29 label C (transactions with specified countries) on the trust or partnership return form.

                Legacy Rule Format:
                    IF [PTR109] = FALSE AND [IDS9] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    [PTR109] = PTR:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
                    ^PTR109 = FALSE AND ^IDS9 = TRUE

        Data Elements:
            
                    RP:^PTR109 = rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            
                    RP:^IDS9 = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    */
        protected void VRATOPTR431001(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431001

            assertion = false; // = (report.PTR109 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = @"If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the IDS then the response at Overseas transactions - Transactions with specified countries on the partnership return must be 'True'.",
                    //Location = "/xbrli:xbrl/rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431001" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR109", Value = GetValueOrEmpty(report.PTR109) });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431001

        #region VR.ATO.PTR.431002

        /*  VR.ATO.PTR.431002
                    An amount is showing at debt deductions in earning non-assessable foreign income claimed under s25-90 (on the IDS) and there is no amount at Interest expenses, Other deductions or Interest expenses overseas

                Legacy Rule Format:
                    IF ([PTR346] = NULL OR [PTR346] = 0 ) AND (ANY OCCURRENCE OF([PTR96] = NULL OR [PTR96] = 0)) AND ([PTR137] = NULL OR [PTR137] = 0 ) AND ([IDS207] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS207] = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
                    [PTR96] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherDeductions)
                    [PTR137] = PTR:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [PTR346] = PTR:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
                    (^PTR346 = NULL OR ^PTR346 = 0) AND ((AnyOccurrence(^PTR96, ^PTR96 = NULL) OR ^PTR96 = 0)) AND (^PTR137 = NULL OR ^PTR137 = 0) AND (^IDS207 > 0)

        Data Elements:
            
                    RP.JAUS:^PTR346 = bafpr2.02.17:Expense.Operating.Interest.Amount
            
                    RP.JAUS:^IDS207 = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
            
                    RP.JAUS:^PTR96 = bafpr1.02.04:Expense.DeductibleOther.Amount
            
                    RP.JFOREIGN:^PTR137 = bafpr1.02.05:Expense.Interest.Amount
                    */
        protected void VRATOPTR431002(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431002

            assertion = false; // = ((report.PTR346 == null || report.PTR346 == 0) && ((report.OtherDeductionsCollection == null ? false : report.OtherDeductionsCollection.Any(PTR96Repeat => PTR96Repeat.PTR96 == null)) || otherDeductions.PTR96 == 0) && (report.PTR137 == null || report.PTR137 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest expenses required",
                    LongDescription = @"If Debt deductions in earning non-assessable foreign income claimed under s25-90 has been completed on the IDS then Total interest expenses or Interest expenses overseas must be greater than zero on the partnership form.",
                    //Location = "/xbrli:xbrl/bafpr2.02.17:Expense.Operating.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431002" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR346", Value = report.PTR346.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR96", Value = "PTR96" });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR137", Value = report.PTR137.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431002

        #region VR.ATO.PTR.431003

        /*  VR.ATO.PTR.431003
                    An amount is showing at Total amount of your share of net income/distribution of profit (on  the IDS) and there is no amount at  Gross other assessable foreign source income

                Legacy Rule Format:
                    IF ([PTR104] = NULL) AND ([IDS219] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS219] = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    [PTR104] = PTR:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
                    (^PTR104 = NULL) AND (^IDS219 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^PTR104 = bafpr1.02.04:Income.Other.Amount
            
                    RP.JFOREIGN:^IDS219 = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    */
        protected void VRATOPTR431003(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431003

            assertion = false; // = (report.PTR104 == null &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross other assessable foreign source income required",
                    LongDescription = @"If Total amount of your share of net income/distribution of profit has been completed on the IDS then the Gross Other assessable foreign source income must be completed on the partnership return form.",
                    //Location = "/xbrli:xbrl/bafpr1.02.04:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431003" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR104", Value = report.PTR104.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431003

        #region VR.ATO.PTR.431014

        /*  VR.ATO.PTR.431014
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Interest expenses overseas

                Legacy Rule Format:
                    IF ([PTR137] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR137] = PTR:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
                    (^PTR137 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^PTR137 = bafpr1.02.05:Expense.Interest.Amount
                    */
        protected void VRATOPTR431014(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431014

            assertion = false; // = (report.PTR137.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431014",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Interest expenses overseas",
                    //Location = "/xbrli:xbrl/bafpr1.02.05:Expense.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431014" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR137", Value = report.PTR137.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431014

        #region VR.ATO.PTR.431015

        /*  VR.ATO.PTR.431015
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Royalty expenses overseas

                Legacy Rule Format:
                    IF ([PTR138] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^PTR138 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
                    */
        protected void VRATOPTR431015(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431015

            assertion = false; // = (report.PTR138.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Royalty expenses overseas",
                    //Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431015" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431015

        #region VR.ATO.PTR.431045

        /*  VR.ATO.PTR.431045
                    If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.

                Legacy Rule Format:
                    IF
                    ([PTR102] = FALSE AND [PTR108] = FALSE AND [PTR277] = FALSE) AND (([PTR137] = NULL OR [PTR137] = 0) AND ([PTR138] = NULL OR [PTR138] = 0)) AND (COUNT (Schedule = “IDS”) = 1)
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR102] = PTR:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
                    [PTR108] = PTR:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    [PTR137] = PTR:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
                    [PTR277] = PTR:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
                    (^PTR102 = FALSE AND ^PTR108 = FALSE AND ^PTR277 = FALSE) AND ((^PTR137 = NULL OR ^PTR137 = 0) AND (^PTR138 = NULL OR ^PTR138 = 0)) AND (CountDocument('IDS') = 1)

        Data Elements:
            
                    RP.JFOREIGN:^PTR102 = bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
            
                    RP:^PTR108 = rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            
                    RP.JFOREIGN:^PTR137 = bafpr1.02.05:Expense.Interest.Amount
            
                    RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
            
                    RP:^PTR277 = bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOPTR431045(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.431045

            assertion = false; // = (report.PTR102 == false && report.PTR108 == false && report.PTR277 == false && ((report.PTR137 == null || report.PTR137 == 0) && (report.PTR138 == null || report.PTR138 == 0)) && @"IDS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not required",
                    LongDescription = @"If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.",
                    //Location = "/xbrli:xbrl/bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431045" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR102", Value = GetValueOrEmpty(report.PTR102) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR108", Value = GetValueOrEmpty(report.PTR108) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR277", Value = GetValueOrEmpty(report.PTR277) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR137", Value = report.PTR137.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.431045

        #region VR.ATO.PTR.432000

        /*  VR.ATO.PTR.432000
                    If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS

                Legacy Rule Format:
                    IF ([PTR251] > 0 OR [PTR252] > 0) 
                    AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS177] <> TRUE)
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS177] = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
                    [PTR251] = PTR:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount
                    [PTR252] = PTR:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount

            Technical Business Rule Format:
                    (^PTR251 > 0 OR ^PTR252 > 0) AND (CountDocument('IDS') = 1 AND ^IDS177 <> TRUE)

        Data Elements:
            
                    RP.TOFA:^PTR251 = bafpr1.02.04:Income.GainsTotal.Amount
            
                    RP.TOFA:^IDS177 = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
            
                    RP.TOFA:^PTR252 = bafpr1.02.04:Expense.LossesTotal.Amount
                    */
        protected void VRATOPTR432000(List<ProcessMessageDocument> response, PTR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.PTR.432000

            assertion = false; // = ((report.PTR251.GetValueOrDefault() > 0 || report.PTR252.GetValueOrDefault() > 0) && (@"IDS") == 1 &&  != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.432000",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total TOFA losses or Total TOFA gains amount must be provided",
                    LongDescription = @"If there is a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS, then there must be an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules",
                    //Location = "/xbrli:xbrl/bafpr1.02.04:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.432000" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR251", Value = report.PTR251.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR252", Value = report.PTR252.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PTR.432000
        #endregion Cross Form Rules - not supported yet
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(PTR2017 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOGEN430008(response, report);
            VRATOGEN438002(response, report);
            VRATOGEN438003(response, report);
            VRATOGEN438017(response, report);
            VRATOPTR430019(response, report);
            VRATOPTR430020(response, report);
            VRATOPTR430025(response, report);
            VRATOPTR430081(response, report);
            VRATOPTR430082(response, report);
            VRATOPTR430083(response, report);
            VRATOPTR430084(response, report);
            VRATOPTR430086(response, report);
            VRATOPTR430088(response, report);
            VRATOPTR430090(response, report);
            VRATOPTR430176(response, report);
            VRATOPTR430178(response, report);
            VRATOPTR430180(response, report);
            VRATOPTR430183(response, report);
            VRATOPTR430995(response, report);
            VRATOPTR430996(response, report);
            VRATOPTR430998(response, report);
            VRATOPTR430999(response, report);
            VRATOPTR431000(response, report);
            VRATOPTR431001(response, report);
            VRATOPTR431002(response, report);
            VRATOPTR431003(response, report);
            VRATOPTR431014(response, report);
            VRATOPTR431015(response, report);
            VRATOPTR431045(response, report);
            VRATOPTR432000(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
